Imports System.Xml

''' <summary>
''' Beschreibung fr eine aus mehreren Abschnitten MP3-Datei
''' </summary>
''' <remarks>enthlt Informationen zum Aufbau einer aus einzelnen Abschnitten MP3-Datei </remarks>
Public Class MP3File
    Inherits List(Of MP3FileSection)

#Region " Fields / Events "

    ''' <summary>
    ''' Name der Radio-Station
    ''' </summary>
    Private m_StationName As String

    ''' <summary>
    ''' Dateiname der zwischengespeicherten MP3-Datei
    ''' </summary>
    Private m_MP3FileName As String

    ''' <summary>
    ''' Start der Aufzeichnung
    ''' </summary>
    Private m_RecordingStart As Date = Now

    ''' <summary>
    ''' EndTime der Aufzeichnung
    ''' </summary>
    Private m_RecordingEnd As Date = Now

    ''' <summary>
    ''' Event, das ausgelst wird, wenn sich der Empfangs-Status dieser Datei gendert hat
    ''' </summary>
    Public Event Receiving_Changed()

    ''' <summary>
    ''' Event, das ausgelst wird, wenn sich die Text-Anzeige dieser Datei gendert hat
    ''' </summary>
    Public Event Text_Changed()

    ''' <summary>
    ''' Referenz zur Parent-Liste mit MP3-Files
    ''' </summary>
    Private m_List As MP3Source

    ''' <summary>
    ''' Text, der zum Anzeigen z.B. in einem TreeNode verwendet wird
    ''' </summary>
    Private m_Text As String = ""
#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor zum Erstellen einer neuen MP3-Dateibersicht 
    ''' </summary>
    ''' <param name="MP3FileName">Datei-Name</param>
    ''' <param name="List">Parent-Liste</param>
    ''' <param name="StationName">Name der Radio-Station</param>
    Public Sub New(ByVal MP3FileName As String, ByVal List As MP3Source, ByVal StationName As String)
        m_List = List
        m_MP3FileName = MP3FileName
        m_StationName = StationName
        CreateText()
    End Sub

    ''' <summary>
    ''' Konstruktor zum Erstellen einer MP3-Dateibersicht mit Informationen aus einem XML-Node
    ''' </summary>
    ''' <param name="Nd">XML-Node</param>
    ''' <param name="List">Parent-Liste</param>
    Public Sub New(ByVal Nd As XmlNode, ByVal List As MP3Source)
        m_List = List
        m_MP3FileName = Nd.Attributes("FileName").Value
        m_StationName = Nd.Attributes("StationName").Value
        m_RecordingStart = New Date(CLng(Nd.Attributes("RecordingStart").Value))
        m_RecordingEnd = New Date(CLng(Nd.Attributes("RecordingEnd").Value))
        For Each xnd As XmlNode In Nd.ChildNodes(0).ChildNodes
            Me.Add(New MP3FileSection(xnd))
        Next
        CreateText()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt zurck, ob die MP3-Datenquelle noch Daten empfngt
    ''' </summary>
    Overridable ReadOnly Property Receiving() As Boolean
        Get
            Return False
        End Get
    End Property


    ''' <summary>
    ''' gibt den Namen der verwendeten Radio-Station zurck oder legt diesen fest
    ''' </summary>
    Property StationName() As String
        Get
            Return m_StationName
        End Get
        Set(ByVal value As String)
            If m_StationName <> value Then
                m_StationName = value
                CreateText()
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt den Dateinamen der aufgezeichneten MP3-Datei zurck
    ''' </summary>
    ReadOnly Property FileName() As String
        Get
            Return m_MP3FileName
        End Get
    End Property

    ''' <summary>
    ''' liefert den Text zur Anzeige in einem TreeNode
    ''' </summary>
    Property Text() As String
        Get
            Return m_Text
        End Get
        Private Set(ByVal value As String)
            If m_Text <> value Then
                m_Text = value
                RaiseEvent Text_Changed()
            End If
        End Set
    End Property


    WriteOnly Property RecordingEnd() As Date
        Set(ByVal value As Date)
            If m_RecordingEnd <> value Then
                m_RecordingEnd = value
                CreateText()
            End If
        End Set
    End Property
#End Region

#Region " Methods "

    ''' <summary>
    ''' Entfernt diese MP3-Dateibersicht aus der Parent-Liste
    ''' </summary>
    Public Sub Delete()
        m_List.Remove(Me)
    End Sub

    ''' <summary>
    ''' schreibt die enthaltenen Informationen in eine XML-Datei
    ''' </summary>
    ''' <param name="Writer">XmlTextWriter</param>
    Public Sub Write_XML(ByVal Writer As XmlTextWriter)
        Writer.WriteAttributeString("FileName", m_MP3FileName)
        Writer.WriteAttributeString("StationName", m_StationName)
        Writer.WriteAttributeString("RecordingStart", m_RecordingStart.Ticks.ToString)
        Writer.WriteAttributeString("RecordingEnd", m_RecordingEnd.Ticks.ToString)
        Writer.WriteStartElement("Sections")
        For Each Section As MP3FileSection In Me
            Writer.WriteStartElement("Section")
            Section.Write_XML(Writer)
            Writer.WriteEndElement()
        Next
        Writer.WriteEndElement()
    End Sub

    ''' <summary>
    ''' Erstellt einen String, der Status und Inhalt der MP3-Datenquelle anzeigt
    ''' </summary>
    Private Sub CreateText()
        Dim Spn As New TimeSpan(m_RecordingEnd.Ticks - m_RecordingStart.Ticks)
        Dim RestSeconds As Integer = Spn.Seconds
        Dim Hours As Integer = Spn.Hours
        Dim Minutes As Integer = Spn.Minutes

        Dim LengthString As String = " (" & My.LanguageInfo.TimeLengthString(Hours, Minutes, RestSeconds) & ")"

        Me.Text = m_StationName & " " & m_RecordingStart.ToShortDateString & " " & m_RecordingStart.ToString("HH:mm:ss") & LengthString
    End Sub

    ''' <summary>
    ''' lst das Receiving_Changed-Event aus
    ''' </summary>
    Public Sub RaiseReceivingChanged()
        RaiseEvent Receiving_Changed()
    End Sub

#End Region

End Class
