Imports System.Xml
''' <summary>
''' Datenquelle fr die im Projekt verwalteten Daten
''' </summary>
Public Class RadioRecorderSource

#Region " Fields / Events "

    ''' <summary>
    ''' Auflistung verwendeter MP3-Dateien
    ''' </summary>
    Private m_MP3Source As MP3Source

    ''' <summary>
    ''' Liste mit Radio-Stationen
    ''' </summary>
    Private m_StationList As RadioStationSource

    ''' <summary>
    ''' Pfad der XML-Datei zum Speichern der Inhalte
    ''' </summary>
    Private m_XMLFilePath As String

    ''' <summary>
    ''' Optionen zum Speichern
    ''' </summary>
    Private m_MP3SaveOptions As New MP3SaveOptions

#End Region

#Region " Constructor "

    Public Sub New()
        ' Pfad zur XML-Datei
        m_XMLFilePath = Application.StartupPath & "\RadioRecorderData.xml"

        If IO.File.Exists(m_XMLFilePath) Then
            ' Wenn eine XML-Datei existiert, deren Inhalte lesen
            Dim Nd As XmlNode = GetXML(m_XMLFilePath)

            m_MP3Source = New MP3Source(Nd.ChildNodes(1))
            m_StationList = New RadioStationSource(Nd.ChildNodes(0), m_MP3Source)
            m_MP3SaveOptions = New MP3SaveOptions(Nd.ChildNodes(2))
            If m_StationList.Count = 0 Then
                AddExampleStations()
            End If
        Else
            ' ohne XML-Datei 
            m_MP3Source = New MP3Source
            m_StationList = New RadioStationSource(m_MP3Source)
            ' Ein paar Beispiel-Stationen hinzufgen
            AddExampleStations()
        End If

    End Sub

    ''' <summary>
    ''' fgt ein paar Beispiel-Radiostationen ein 
    ''' </summary>
    Private Sub AddExampleStations()
        m_StationList.AddSingleStation("http://scfire-nyk-aa05.stream.aol.com:80/stream/1040", ".977 The 80s Channel")
        m_StationList.AddSingleStation("http://scfire-dll-aa02.stream.aol.com:80/stream/1010", "S K Y . F M")
        m_StationList.AddSingleStation("http://scfire-chi-aa01.stream.aol.com:80/stream/1048", "Radio Paradise")
        m_StationList.AddSingleStation("http://72.64.217.165:8000", "javaBeats.FM Internet Radio")
    End Sub
#End Region

#Region " Properties "

    ''' <summary>
    ''' Auflistung verwendeter MP3-Dateien
    ''' </summary>
    ReadOnly Property MP3Source() As MP3Source
        Get
            Return m_MP3Source
        End Get
    End Property

    ''' <summary>
    ''' Optionen zum Speichern
    ''' </summary>
    ReadOnly Property MP3SaveOptions() As MP3SaveOptions
        Get
            Return m_MP3SaveOptions
        End Get
    End Property

    ''' <summary>
    ''' Liste mit Radio-Stationen
    ''' </summary>
    ReadOnly Property RadioStations() As RadioStationSource
        Get
            Return m_StationList
        End Get
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' speichert den Inhalt als XML-Datei
    ''' </summary>
    Public Sub Save_XML()
        Dim Writer As XmlTextWriter = GetWriter(m_XMLFilePath)
        Writer.WriteStartElement("RadioStations")
        m_StationList.Write_XML(Writer)
        Writer.WriteEndElement()
        Writer.WriteStartElement("MP3Files")
        m_MP3Source.Write_XML(Writer)
        Writer.WriteEndElement()
        Writer.WriteStartElement("MP3SaveOptions")
        m_MP3SaveOptions.Write_XML(Writer)
        Writer.WriteEndElement()
        CloseWriter(Writer)
    End Sub

    ''' <summary>
    ''' liefert eine XmlTextWriter zum Schreiben einer XML-Datei fr den angegebenen Pfad
    ''' </summary>
    Private Shared Function GetWriter(ByVal Pfad As String) As XmlTextWriter
        Dim Writer As New XmlTextWriter(Pfad, System.Text.Encoding.UTF8)
        Writer.WriteStartDocument(True)

        Writer.WriteStartElement("Elemente")
        Writer.WriteStartElement("Content")
        Return Writer
    End Function

    ''' <summary>
    ''' schliesst den zuvor geffneten XmlTextWriter 
    ''' </summary>
    Private Shared Sub CloseWriter(ByVal Writer As XmlTextWriter)
        Writer.WriteEndElement()
        Writer.WriteEndElement()
        Writer.WriteEndDocument()
        Writer.Close()
    End Sub

    ''' <summary>
    ''' liefert den XML-Inhalt einer existierenden XML-Datei 
    ''' </summary>
    Private Shared Function GetXML(ByVal Pfad As String) As XmlNode
        Dim reader As XmlTextReader = New XmlTextReader(Pfad)

        reader.WhitespaceHandling = WhitespaceHandling.None
        Dim xmlDoc As XmlDocument = New XmlDocument
        xmlDoc.Load(reader)
        reader.Close()

        Dim xnod As XmlNode = xmlDoc.DocumentElement

        Return xnod.ChildNodes(0)
    End Function

#End Region

End Class

