Imports System.Xml

''' <summary>
''' enthlt Informationen zu einer Internet-Radiostation und startet bzw. beendet 
''' das Aufzeichnen von MP3-Dateien
''' </summary>
''' <remarks></remarks>
Public Class RadioStation

#Region " Fields / Events "

    ''' <summary>
    ''' URL der Radio-Station
    ''' </summary>
    Private m_URL As String

    ''' <summary>
    ''' Name der Radio-Station
    ''' </summary>
    Private m_StationName As String = ""

    ''' <summary>
    ''' Referenz zur Auflistung mit MP3-Dateien
    ''' </summary>
    Private m_MP3Source As MP3Source

    ''' <summary>
    ''' Liste mit Auftrgen zum Aufzeichnen der Radio-Station
    ''' </summary>
    Private WithEvents m_RecordingTasks As RecordingTaskList

    ''' <summary>
    ''' Referenz zum Empnger fr diese Radiostation 
    ''' </summary>
    Private WithEvents m_Receiver As RadioStationReceiver

    ''' <summary>
    ''' Event beim ndern des Empfangsstatus
    ''' </summary>
    Public Event Recording_Changed()

    ''' <summary>
    ''' Event bei nderung des angezeigten Textes
    ''' </summary>
    Public Event Text_Changed()

    ''' <summary>
    ''' Angezeigter Text
    ''' </summary>
    Private m_Text As String = ""

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor mit URL und MP3-Listen-Quelle
    ''' </summary>
    Public Sub New(ByVal URL As String, ByVal MP3Source As MP3Source)
        m_MP3Source = MP3Source
        m_URL = URL
        m_StationName = m_URL
        m_RecordingTasks = New RecordingTaskList
        RefreshText()
    End Sub

    ''' <summary>
    ''' Konstruktor mit XmlNode und MP3-Listen-Quelle zum Lesen der Infos zur RadioStation aus einer XML-Datei
    ''' </summary>
    Public Sub New(ByVal Nd As XmlNode, ByVal MP3Source As MP3Source)
        m_MP3Source = MP3Source
        m_URL = Nd.Attributes("URL").Value
        m_StationName = Nd.Attributes("StationName").Value
        m_RecordingTasks = New RecordingTaskList(Nd.ChildNodes(0))
        RefreshText()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Angezeigter Text
    ''' </summary>
    Property Text() As String
        Get
            Return m_Text
        End Get
        Private Set(ByVal value As String)
            If m_Text <> value Then
                m_Text = value
                RaiseEvent Text_Changed()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Referenz zur Auflistung mit MP3-Dateien
    ''' </summary>
    ReadOnly Property MP3Source() As MP3Source
        Get
            Return m_MP3Source
        End Get
    End Property

    ''' <summary>
    ''' Auflistung mit Auftrgen zum Aufzeichnen der Radio-Station
    ''' </summary>
    ReadOnly Property RecordingTasks() As RecordingTaskList
        Get
            Return m_RecordingTasks
        End Get
    End Property


    ''' <summary>
    ''' liefert die URL der Radio-Station
    ''' </summary>
    ReadOnly Property URL() As String
        Get
            Return m_URL
        End Get
    End Property

    ''' <summary>
    ''' gibt zurck oder legt fest, ob fr diese Radio-Station eine MP3-Datei aufgezeichnet wird
    ''' </summary>
    Property IsRecording() As Boolean
        Get
            Return m_RecordingTasks.IsRecording
        End Get
        Set(ByVal value As Boolean)
            m_RecordingTasks.ManualRecording = value
        End Set
    End Property

    ''' <summary>
    ''' Name der Radio-Station
    ''' </summary>
    Property StationName() As String
        Get
            Return m_StationName
        End Get
        Set(ByVal value As String)
            If m_StationName <> value Then
                m_StationName = value
                RefreshText()
            End If
        End Set
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' Beendet alle Aufzeichnungen
    ''' </summary>
    Public Sub StopAllRecording()
        If m_Receiver IsNot Nothing Then
            m_Receiver.Dispose()
        End If
    End Sub

    ''' <summary>
    ''' schreibt Informationen zur RadioStation in eine XML-Datei
    ''' </summary>
    Public Sub Write_XML(ByVal Writer As XmlTextWriter)
        Writer.WriteAttributeString("URL", m_URL)
        Writer.WriteAttributeString("StationName", m_StationName)
        Writer.WriteStartElement("RecordingTasks")
        m_RecordingTasks.Write_XML(Writer)
        Writer.WriteEndElement()
    End Sub

    ''' <summary>
    ''' Aktualisiert den angezeigten Text
    ''' </summary>
    Private Sub RefreshText()
        Dim Txt As String = ""

        If m_Receiver Is Nothing OrElse m_Receiver.ReceiveState = ReceiveStates.Inactive Then
            Txt = m_StationName
        Else
            If m_Receiver.ReceiveState = ReceiveStates.Starting Then
                Txt = My.LanguageInfo.TextStarting & ": "
            Else
                Txt = My.LanguageInfo.TextRecording & ": "
            End If

            Txt &= m_StationName

            Dim CurrentTitle As String = m_Receiver.CurrentTitle
            If CurrentTitle <> "" Then
                Txt &= " " & My.LanguageInfo.TextCurrentTitle & ": " & CurrentTitle
            End If
        End If

        Me.Text = Txt
    End Sub

    ''' <summary>
    ''' Handler bei nderung des Aufzeichnungs-Zustands in der Quelle fr Aufzeichnungs-Auftrge
    ''' </summary>
    Private Sub m_RecordingTasks_Recording_Changed() Handles m_RecordingTasks.Recording_Changed

        If m_Receiver IsNot Nothing Then
            m_Receiver.Dispose()
            m_Receiver = Nothing
        End If

        If m_RecordingTasks.IsRecording Then
            m_Receiver = New RadioStationReceiver(Me)
            m_Receiver.Execute(m_URL, True)
        End If

        RefreshText()
        RaiseEvent Recording_Changed()
    End Sub

    ''' <summary>
    ''' Handler bei nderung des aktuellen Titels im Radio-Empfnger
    ''' </summary>
    Private Sub m_Receiver_CurrentTitle_Changed() Handles m_Receiver.CurrentTitle_Changed
        RefreshText()
    End Sub

    ''' <summary>
    ''' Handler bei Erstellung einer neuen Datei im Radio-Empfnger
    ''' </summary>
    Private Sub m_Receiver_File_Changed() Handles m_Receiver.File_Created
        RefreshText()
        RaiseEvent Recording_Changed()
    End Sub

    ''' <summary>
    ''' Handler bei nderung des Empfangsstatus des Radio-Empfngers
    ''' </summary>
    Private Sub m_Receiver_ReceiveState_Changed() Handles m_Receiver.ReceiveState_Changed
        RefreshText()
        RaiseEvent Recording_Changed()
    End Sub

    ''' <summary>
    ''' Handler bei Autreten eines Fehlers im Radio-Empfnger
    ''' </summary>
    Private Sub m_Receiver_Was_Error() Handles m_Receiver.Was_Error
        m_RecordingTasks.WasError = True
    End Sub

#End Region

End Class
