Imports System.Net
Imports System.Net.Sockets
Imports System.Text

''' <summary>
''' Empfngt Daten eines Shoutcast-Streams fr eine URL im Netzwerk
''' </summary>
Public Class StreamReceiver
    Implements IDisposable

#Region " Fields / Events "

    ''' <summary>
    ''' Empfangsstatus
    ''' </summary>
    Private m_ReceiveState As ReceiveStates = ReceiveStates.Inactive

    ''' <summary>
    ''' Liste mit MP3-Abschnitten
    ''' </summary>
    Private m_ReceiveFile As MP3ReceivingFile

    ''' <summary>
    ''' Objekt zum Sammeln von Bytes und anderer Daten beim Empfang eines MP3-Streams
    ''' </summary>
    Private WithEvents m_ByteList As ReceivedByteList

    ''' <summary>
    ''' Flag mit Info, ob gerade Bytes fr einen Musik-Abschnitt gelesen wurden
    ''' </summary>
    Private m_MusicWritten As Boolean = False

    ''' <summary>
    ''' Event bei nderung des Empfangs-Status
    ''' </summary>
    Public Event ReceiveState_Changed()

    ''' <summary>
    ''' Event bei Auftreten eines Fehlers
    ''' </summary>
    Public Event Was_Error()

    ''' <summary>
    ''' Event bei Notwendigkeit zum Wechsel der URL
    ''' </summary>
    Public Event MustRedirect(ByVal NewURL As String)

    ''' <summary>
    ''' URL zum Shoutcast-Server
    ''' </summary>
    Private m_Url As String

    ''' <summary>
    ''' Socket zum bertragen der Daten
    ''' </summary>
    Private m_Socket As Socket

    ''' <summary>
    ''' Flag mit Info, ob Dispose bereits einmal durchgefhrt wurde
    ''' </summary>
    Private m_Disposed As Boolean = False

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Default Konstruktor
    ''' </summary>
    ''' <param name="Url">URL zum Shoutcast-Server</param>
    ''' <param name="ByteList">Objekt zum Sammeln von Bytes und anderer Daten beim Empfang eines MP3-Streams</param>
    ''' <param name="ReceiveFile">Liste zum Speichern der MP3-Abschnitte</param>
    Public Sub New(ByVal Url As String, ByVal ByteList As ReceivedByteList, ByVal ReceiveFile As MP3ReceivingFile)
        m_Url = Url
        m_ByteList = ByteList
        m_ReceiveFile = ReceiveFile
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Empfangsstatus
    ''' </summary>
    Property ReceiveState() As ReceiveStates
        Get
            Return m_ReceiveState
        End Get
        Private Set(ByVal value As ReceiveStates)
            If m_ReceiveState <> value Then
                m_ReceiveState = value

                RaiseEvent ReceiveState_Changed()
            End If
        End Set
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' stellt eine Socket-Verbindung zum Server des Shoutcast-Streams her
    ''' </summary>
    Private Sub Connect()

        ' Host-Name der Radiostation
        Dim HostName As String = ""
        ' Der Port des verwendeten URI, ber den der Stream geliefert wird 
        Dim Port As Integer

        Try
            Dim TmpUri As New Uri(m_Url)
            HostName = TmpUri.DnsSafeHost
            Port = TmpUri.Port
            m_Url = TmpUri.AbsolutePath
        Catch ex As Exception

        End Try


        ' versuchen, die IP-Adresse des Servers festzustellen
        Dim address As IPAddress = Nothing

        Try
            address = IPAddress.Parse(HostName)
        Catch ex As FormatException
            Try
                ' Versuchen, die DNS-Adresse als IP-Adresse aufzulsen
                address = Dns.GetHostEntry(HostName).AddressList(0)
            Catch ex1 As Exception
                Throw ex1
            End Try
        Catch ex As Exception
            Throw ex
        End Try


        ' Socket fr die Verbindung erstellen
        m_Socket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP)
        m_Socket.Connect(New IPEndPoint(address, Port))

        ' TimeOut festlegen, damit der Empfang beendet wird, wenn keine Daten mehr vom Server kommen
        m_Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, 5000)


        ' Nun den initialen Get-Request senden
        Dim Request As Byte() = Encoding.ASCII.GetBytes( _
        "GET " & m_Url & " HTTP/1.0" & vbCrLf & "Host: " & HostName & vbCrLf & _
        "User-Agent: WinampMPEG/5.0" & vbCrLf & "Accept: */*" & vbCrLf & _
        "Icy-MetaData:1" & vbCrLf & vbCrLf)

        m_Socket.Send(Request, 0, Request.Length, SocketFlags.None)

        ' Temporrer Puffer fr den Empfang
        Dim tmpBuffer(1023) As Byte
        ' Response als String
        Dim ReceivedText As String = ""

        ' So lange empfangen, bis der empfangene Text vbCrLf & vbCrLf enthlt
        While ReceivedText.IndexOf(vbCrLf & vbCrLf) = -1
            Dim count As Integer = m_Socket.Receive(tmpBuffer, 0, tmpBuffer.Length, SocketFlags.None)
            ReceivedText &= Encoding.ASCII.GetString(tmpBuffer, 0, count)
        End While

        ' Untersucht die erste Textzeile der Antwort 
        Dim FirstLine As String = ReceivedText.Substring(0, ReceivedText.IndexOf(vbCrLf))
        If FirstLine.IndexOf("200 OK") > -1 Then
            ' Antwort OK - weitere Daten (Name der Radiostation, Bitrate, Metaintervall) knnen gelesen werden
            GetTransmitVars(ReceivedText)
        ElseIf FirstLine.IndexOf("302 Found") > -1 Then
            ' Antwort Redirect - es muss eine Verbindung zu einer anderen URL aufgebaut werden
            m_ByteList.IsValid = False
            RaiseEvent MustRedirect(SharedFunctions.GetSubString(ReceivedText, "Location: ", vbCrLf))
            Exit Sub
        Else
            Throw New Exception("Header Error")
        End If

        ' Manchmal schlieen sich Bytes mit Informationen direkt an den Header an, diese mssen hier glesen werden
        If Not ReceivedText.EndsWith(vbCrLf & vbCrLf) Then
            ' Daten folgen auf vbCrLf & vbCrLf
            m_ByteList.AddRange(Encoding.ASCII.GetBytes(ReceivedText.Substring(ReceivedText.IndexOf(vbCrLf & vbCrLf) + 4)))
        End If
    End Sub

    ''' <summary>
    ''' liefert Eigenschaften zum Shoutcast-Stream aus dem Header 
    ''' </summary>
    ''' <param name="ReceivedText">String, der alle Header-Informationen enthlt</param>
    Private Sub GetTransmitVars(ByVal ReceivedText As String)
        ' Meta-Intervall
        Dim tmpText As String = SharedFunctions.GetSubString(ReceivedText, "icy-metaint:", vbCrLf)
        If tmpText.Length <> 0 Then
            m_ByteList.MetaInterval = CInt(tmpText)
        End If

        ' Name der Radiostation
        Dim StationName As String = SharedFunctions.GetSubString(ReceivedText, "icy-name:", vbCrLf).Trim

        Dim Idx1 As Integer = StationName.IndexOf("-")
        Dim Idx2 As Integer = StationName.IndexOf(":")

        If Idx1 > -1 Then
            If Idx2 > -1 Then
                StationName = StationName.Substring(0, Math.Min(Idx1, Idx2)).Trim
            Else
                StationName = StationName.Substring(0, Idx1).Trim
            End If
        Else
            If Idx2 > -1 Then
                StationName = StationName.Substring(0, Idx2).Trim
            End If
        End If

        m_ByteList.StationName = StationName

        ' Bitrate
        tmpText = SharedFunctions.GetSubString(ReceivedText, "icy-br:", vbCrLf)
        If tmpText.Length <> 0 Then
            m_ByteList.BitRate = CInt(tmpText)
        End If
    End Sub


    ''' <summary>
    ''' startet dem Empfang
    ''' </summary>
    Public Sub Receive()

        Try
            Me.ReceiveState = ReceiveStates.Starting
            ' Socket-Verbindung herstellen
            Connect()

            Me.ReceiveState = ReceiveStates.Recording

            ' Byte-Puffer
            Dim WorkBytes(2047) As Byte

            While m_ByteList.IsValid
                ' Daten empfangen
                Dim count As Integer = m_Socket.Receive(WorkBytes, 0, WorkBytes.Length, SocketFlags.None)
                Dim data(count - 1) As Byte
                Buffer.BlockCopy(WorkBytes, 0, data, 0, count)
                m_ByteList.AddRange(data)
            End While

        Catch ex As Exception
            RaiseEvent Was_Error()
        Finally
            ' Socket schliessen
            If m_Socket IsNot Nothing Then
                If m_Socket.Connected Then
                    m_Socket.Shutdown(SocketShutdown.Both)
                End If
                m_Socket.Close()
                m_Socket = Nothing
            End If
            Me.ReceiveState = ReceiveStates.Inactive
        End Try

    End Sub

    ''' <summary>
    ''' Handler fr das HaveDataToWrite-Event der ReceivedByteList
    ''' </summary>
    ''' <remarks>liefert entweder Text fr Titel und Interpret oder Musik-Bytes in Meta-Intervall-Lnge</remarks>
    Private Sub m_ByteList_HaveDataToWrite() Handles m_ByteList.HaveDataToWrite
        If Not m_MusicWritten Then
            Dim Bytes() As Byte = m_ByteList.GetMusicBytes
            m_ReceiveFile.Write(Bytes, Bytes.Length)
            ' Flag auf True setzen
            m_MusicWritten = True
        Else
            Dim txt As String = m_ByteList.GetText
            If txt <> "" Then
                Debug.WriteLine(txt)
                m_ReceiveFile.CurrentTitle = SharedFunctions.GetSubString(txt, "='", "';")
            End If
            ' Flag auf False setzen
            m_MusicWritten = False
        End If
    End Sub

    ''' <summary>
    ''' Einbinden von IDisposable
    ''' </summary>
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.m_Disposed Then

            If disposing Then
                ' Socket schliessen
                If m_Socket IsNot Nothing Then
                    If m_Socket.Connected Then
                        m_Socket.Shutdown(SocketShutdown.Both)
                    End If
                    m_Socket.Close()
                    m_Socket = Nothing
                End If
            End If

        End If
        Me.m_Disposed = True
    End Sub

    ''' <summary>
    ''' Einbinden von IDisposable
    ''' </summary>
    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

#End Region

End Class



