Imports System.Xml

''' <summary>
''' Liste mit Aufzeichnungs-Aufrgen fr eine Radiostation
''' </summary>
''' <remarks>berprft jede halbe Sekunde den Aufzeichnungs-Status und liefert
'''  in diesem Abstand ein Event mit dem aktuellen Auzeichnungs-Status</remarks>
Public Class RecordingTaskList
    Inherits List(Of RecordingTask)

#Region " Fields / Events "

    ''' <summary>
    ''' Timer, mit dem im Halbsekundenabstand berprft wird, ob eine Aufgabe aktiv ist
    ''' </summary>
    Private WithEvents m_Timer As Timer

    ''' <summary>
    ''' Info, ob ein Fehler beim Aufzeichnen aufgetreten ist
    ''' </summary>
    Private m_WasError As Boolean = False

    ''' <summary>
    ''' Info, ob aktuell aufgezeichnet werden muss
    ''' </summary>
    Private m_IsRecording As Boolean = False

    ''' <summary>
    ''' Info, ob "manuell" aufgezeichnet wird
    ''' </summary>
    Private m_ManualRecording As Boolean = False

    ''' <summary>
    ''' Event bei nderung des Aufzeichnungs-Zustands
    ''' </summary>
    Public Event Recording_Changed()

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Parameterloser Konstruktor
    ''' </summary>
    Public Sub New()
        StartTimer()
    End Sub

    ''' <summary>
    ''' Konstruktor fr das Lesen einer XML-Struktur
    ''' </summary>
    ''' <param name="Nd">XML-Node</param>
    Public Sub New(ByVal Nd As XmlNode)
        For Each xNd As XmlNode In Nd.ChildNodes
            If xNd.Name = "SingleTask" Then
                Me.Add(New RecordingTaskSingle(xNd))
            ElseIf xNd.Name = "PeriodicTask" Then
                Me.Add(New RecordingTaskPeriodic(xNd))
            End If
        Next
        StartTimer()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' legt fest, ob ein Fehler beim Aufzeichnen aufgetreten ist
    ''' </summary>
    WriteOnly Property WasError() As Boolean
        Set(ByVal value As Boolean)
            If m_WasError <> value Then
                m_WasError = value
                If m_WasError Then
                    m_Timer.Stop()
                    m_ManualRecording = False
                    IsRecording = False
                End If
            End If
        End Set
    End Property

    ''' <summary>
    ''' legt fest oder gibt zurck, ob aktuell aufgezeichnet werden muss
    ''' </summary>
    Property IsRecording() As Boolean
        Get
            Return m_IsRecording
        End Get
        Private Set(ByVal value As Boolean)
            If m_IsRecording <> value Then
                m_IsRecording = value
                RaiseEvent Recording_Changed()
            End If
        End Set
    End Property

    ''' <summary>
    ''' legt fest, ob "manuell" aufgezeichnet wird
    ''' </summary>
    WriteOnly Property ManualRecording() As Boolean
        Set(ByVal value As Boolean)
            If m_ManualRecording <> value Then
                m_ManualRecording = value
                If m_ManualRecording Then
                    m_WasError = False
                    m_Timer.Start()
                Else
                    m_Timer.Stop()
                    IsRecording = False
                End If
            ElseIf m_IsRecording AndAlso value = False Then
                m_Timer.Stop()
                IsRecording = False
            End If
        End Set
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' Legt die Auftrge zum Auzeichnen neu fest
    ''' </summary>
    ''' <param name="Tasks">Array mit Autrgen zum Auzeichnen</param>
    Public Sub SetTasks(ByVal Tasks() As RecordingTask)
        m_Timer.Stop()
        Me.Clear()
        Me.AddRange(Tasks)
        m_Timer.Start()
    End Sub

    ''' <summary>
    ''' Erstellt und startet einen Timer
    ''' </summary>
    Public Sub StartTimer()
        m_Timer = New Timer
        m_Timer.Interval = 500
        m_Timer.Start()
    End Sub

    ''' <summary>
    ''' schreibt den Inhalt der Liste in eine XML-Struktur
    ''' </summary>
    Public Sub Write_XML(ByVal Writer As XmlWriter)
        For Each Auftr As RecordingTask In Me
            Auftr.Write_XML(Writer)
        Next
    End Sub

    ''' <summary>
    ''' Handler fr das Tick-Event des Timers
    ''' </summary>
    Private Sub m_Timer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_Timer.Tick

        ' nach jedem Intervall berprfen, ob noch aufgezeichnet werden muss
        If m_ManualRecording Then
            IsRecording = True
        Else
            Dim Recording As Boolean = False
            Dim AcTime As Date = Now
            For Each Task As RecordingTask In Me
                If Task.IsActive(AcTime) Then
                    Recording = True
                    Exit For
                End If
            Next
            IsRecording = Recording
        End If

    End Sub

#End Region

End Class
