Imports System.Xml
''' <summary>
''' periodisch wiederkehrende Aufzeichnungs-Aufgabe
''' </summary>
Public Class RecordingTaskPeriodic
    Inherits RecordingTask

#Region " Fields / Events "

    ''' <summary>
    ''' Start-Zeitpunkt
    ''' </summary>
    Private m_StartTime As Date

    ''' <summary>
    ''' End-Zeitpunkt
    ''' </summary>
    Private m_EndTime As Date

    ''' <summary>
    ''' Dauer der Aufzeichnung in Minuten
    ''' </summary>
    Private m_MinutesLength As Integer

    ''' <summary>
    ''' Dictionary mit Wochentagen und Boolean-Wert
    ''' </summary>
    Private m_Weekdays As New Dictionary(Of DayOfWeek, Boolean)

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Standard-Konstruktor
    ''' </summary>
    Public Sub New()
        m_StartTime = Now
        m_EndTime = m_StartTime.AddHours(1)
        ' Dictionary mit Wochentagen fllen
        CreateWeekdays()
        ' Dauer in Minuten bestimmen
        GetMinutesLength()
    End Sub

    ''' <summary>
    ''' erstellt eine Kopie einer RecordingTaskPeriodic-Instanz
    ''' </summary>
    Public Sub New(ByVal Original As RecordingTaskPeriodic)
        m_StartTime = Original.StartTime
        m_EndTime = Original.EndTime
        CreateWeekdays()
        For I As Integer = 0 To 6
            Dim Wd As DayOfWeek = CType(I, DayOfWeek)
            m_Weekdays(I) = Original.Weekdays(Wd)
        Next
        GetMinutesLength()
    End Sub

    ''' <summary>
    ''' Konstruktor zum Erstellen einer RecordingTaskPeriodic-Instanz beim Lesen einer XML-Datei
    ''' </summary>
    Public Sub New(ByVal Nd As XmlNode)
        m_StartTime = New DateTime(CLng(Nd.Attributes("StartTime").Value))
        m_EndTime = New DateTime(CLng(Nd.Attributes("EndTime").Value))

        CreateWeekdays()

        m_Weekdays(DayOfWeek.Monday) = CBool(Nd.Attributes("Monday").Value)
        m_Weekdays(DayOfWeek.Tuesday) = CBool(Nd.Attributes("Tuesday").Value)
        m_Weekdays(DayOfWeek.Wednesday) = CBool(Nd.Attributes("Wednesday").Value)
        m_Weekdays(DayOfWeek.Thursday) = CBool(Nd.Attributes("Thursday").Value)
        m_Weekdays(DayOfWeek.Friday) = CBool(Nd.Attributes("Friday").Value)
        m_Weekdays(DayOfWeek.Saturday) = CBool(Nd.Attributes("Saturday").Value)
        m_Weekdays(DayOfWeek.Sunday) = CBool(Nd.Attributes("Sunday").Value)
        GetMinutesLength()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Dictionary mit Wochentagen und Boolean-Wert, der angibt, ob dieser Tag ausgewhlt wurde
    ''' </summary>
    ReadOnly Property Weekdays() As Dictionary(Of DayOfWeek, Boolean)
        Get
            Return m_Weekdays
        End Get
    End Property

    ''' <summary>
    ''' Start-Zeitpunkt
    ''' </summary>
    Property StartTime() As Date
        Get
            Return m_StartTime
        End Get
        Set(ByVal value As Date)
            If m_StartTime <> value Then
                m_StartTime = value
                GetMinutesLength()
                MyBase.RaiseChanged()
            End If
        End Set
    End Property

    ''' <summary>
    ''' End-Zeitpunkt
    ''' </summary>
    Property EndTime() As Date
        Get
            Return m_EndTime
        End Get
        Set(ByVal value As Date)
            If m_EndTime <> value Then
                m_EndTime = value
                GetMinutesLength()
                MyBase.RaiseChanged()
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt an oder legt fest, ob Montags aufgezeichnet werden soll
    ''' </summary>
    Property Monday() As Boolean
        Get
            Return m_Weekdays(DayOfWeek.Monday)
        End Get
        Set(ByVal value As Boolean)
            m_Weekdays(DayOfWeek.Monday) = value
            MyBase.RaiseChanged()
        End Set
    End Property

    ''' <summary>
    ''' gibt an oder legt fest, ob Dienstags aufgezeichnet werden soll
    ''' </summary>
    Property Tuesday() As Boolean
        Get
            Return m_Weekdays(DayOfWeek.Tuesday)
        End Get
        Set(ByVal value As Boolean)
            m_Weekdays(DayOfWeek.Tuesday) = value
            MyBase.RaiseChanged()
        End Set
    End Property

    ''' <summary>
    ''' gibt an oder legt fest, ob Mittwochs aufgezeichnet werden soll
    ''' </summary>
    Property Wednesday() As Boolean
        Get
            Return m_Weekdays(DayOfWeek.Wednesday)
        End Get
        Set(ByVal value As Boolean)
            m_Weekdays(DayOfWeek.Wednesday) = value
            MyBase.RaiseChanged()
        End Set
    End Property

    ''' <summary>
    ''' gibt an oder legt fest, ob Donnerstags aufgezeichnet werden soll
    ''' </summary>
    Property Thursday() As Boolean
        Get
            Return m_Weekdays(DayOfWeek.Thursday)
        End Get
        Set(ByVal value As Boolean)
            m_Weekdays(DayOfWeek.Thursday) = value
            MyBase.RaiseChanged()
        End Set
    End Property

    ''' <summary>
    ''' gibt an oder legt fest, ob Freitags aufgezeichnet werden soll
    ''' </summary>
    Property Friday() As Boolean
        Get
            Return m_Weekdays(DayOfWeek.Friday)
        End Get
        Set(ByVal value As Boolean)
            m_Weekdays(DayOfWeek.Friday) = value
            MyBase.RaiseChanged()
        End Set
    End Property

    ''' <summary>
    ''' gibt an oder legt fest, ob Samstags aufgezeichnet werden soll
    ''' </summary>
    Property Saturday() As Boolean
        Get
            Return m_Weekdays(DayOfWeek.Saturday)
        End Get
        Set(ByVal value As Boolean)
            m_Weekdays(DayOfWeek.Saturday) = value
            MyBase.RaiseChanged()
        End Set
    End Property

    ''' <summary>
    ''' gibt an oder legt fest, ob Sonntags aufgezeichnet werden soll
    ''' </summary>
    Property Sunday() As Boolean
        Get
            Return m_Weekdays(DayOfWeek.Sunday)
        End Get
        Set(ByVal value As Boolean)
            m_Weekdays(DayOfWeek.Sunday) = value
            MyBase.RaiseChanged()
        End Set
    End Property

    ''' <summary>
    ''' zum Identifizieren ohne Reflection
    ''' </summary>
    Public Overrides ReadOnly Property Type() As RecordingTaskTypes
        Get
            Return RecordingTaskTypes.PeriodicTask
        End Get
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' liefert die Dauer in Minuten und sorgt dafr, dass die Aufzeichnungsdauer 
    ''' hchstens 24 Stunden betrgt
    ''' </summary>
    Private Sub GetMinutesLength()
        Dim StartMinutes As Integer = m_StartTime.Hour * 60 + m_StartTime.Minute
        Dim EndMinutes As Integer = m_EndTime.Hour * 60 + m_EndTime.Minute

        While StartMinutes > EndMinutes
            ' Start um einen Tag zurck verschieben
            StartMinutes -= 1440
        End While
        m_MinutesLength = EndMinutes - StartMinutes
    End Sub

    ''' <summary>
    ''' Fgt dem Wochentags-Dictionary gltige Objekte hinzu
    ''' </summary>
    Private Sub CreateWeekdays()
        m_Weekdays.Add(DayOfWeek.Monday, False)
        m_Weekdays.Add(DayOfWeek.Tuesday, False)
        m_Weekdays.Add(DayOfWeek.Wednesday, False)
        m_Weekdays.Add(DayOfWeek.Thursday, False)
        m_Weekdays.Add(DayOfWeek.Friday, False)
        m_Weekdays.Add(DayOfWeek.Saturday, False)
        m_Weekdays.Add(DayOfWeek.Sunday, False)
    End Sub

    ''' <summary>
    ''' schreibt den Inhalt in eine XML-Datei
    ''' </summary>
    Public Overrides Sub Write_XML(ByVal Writer As XmlTextWriter)
        Writer.WriteStartElement("PeriodicTask")

        Writer.WriteAttributeString("StartTime", m_StartTime.Ticks.ToString)
        Writer.WriteAttributeString("EndTime", m_EndTime.Ticks.ToString)

        Writer.WriteAttributeString("Monday", m_Weekdays(DayOfWeek.Monday).ToString)
        Writer.WriteAttributeString("Tuesday", m_Weekdays(DayOfWeek.Tuesday).ToString)
        Writer.WriteAttributeString("Wednesday", m_Weekdays(DayOfWeek.Wednesday).ToString)
        Writer.WriteAttributeString("Thursday", m_Weekdays(DayOfWeek.Thursday).ToString)
        Writer.WriteAttributeString("Friday", m_Weekdays(DayOfWeek.Friday).ToString)
        Writer.WriteAttributeString("Saturday", m_Weekdays(DayOfWeek.Saturday).ToString)
        Writer.WriteAttributeString("Sunday", m_Weekdays(DayOfWeek.Sunday).ToString)

        Writer.WriteEndElement()
    End Sub

    ''' <summary>
    ''' gibt zurck, ob diese Aufgabe zum angegebenen Zeitpunkt aktiv ist
    ''' </summary>
    Public Overrides Function IsActive(ByVal Time As Date) As Boolean
        Dim TodayDate As DateTime = Time.Date
        Dim YesterdayDate As DateTime = TodayDate.AddDays(-1)

        If m_Weekdays(YesterdayDate.DayOfWeek) = True Then
            Dim StartTime As Date = YesterdayDate.AddHours(m_StartTime.Hour)
            StartTime = StartTime.AddMinutes(m_StartTime.Minute)

            Dim EndTime As Date = StartTime.AddMinutes(m_MinutesLength)
            StartTime = StartTime.AddSeconds(-3)
            EndTime = EndTime.AddSeconds(3)
            If StartTime <= Time AndAlso EndTime >= Time Then
                Return True
            End If
        End If
        If m_Weekdays(TodayDate.DayOfWeek) = True Then
            Dim StartTime As Date = TodayDate.AddHours(m_StartTime.Hour)
            StartTime = StartTime.AddMinutes(m_StartTime.Minute)
            Dim EndTime As Date = StartTime.AddMinutes(m_MinutesLength)
            StartTime = StartTime.AddSeconds(-3)
            EndTime = EndTime.AddSeconds(3)
            If StartTime <= Time AndAlso EndTime >= Time Then
                Return True
            End If
        End If
        Return False
    End Function

    ''' <summary>
    ''' erstellt eine neue RecordingTask-Instanz mit gleichem Inhalt
    ''' </summary>
    Public Overrides Function CreateClone() As RecordingTask
        Return New RecordingTaskPeriodic(Me)
    End Function

    ''' <summary>
    ''' liefert ein Control zum Bearbeiten der aktuellen RecordingTask-Instanz
    ''' </summary>
    Public Overrides Function EditControl() As System.Windows.Forms.Control
        Return New CtlPeriodicRecorderTask(Me)
    End Function

    ''' <summary>
    ''' liefert einen String zum Anzeigen in Controls
    ''' </summary>
    Public Overrides Function ToString() As String
        Dim DayString As String = ""

        For Each Day As DayOfWeek In m_Days
            If m_Weekdays(Day) Then DayString &= m_DayStrings(Day)
        Next

        If DayString <> "" Then
            DayString = " (" & DayString.Substring(2) & ")"
        End If


        Dim Minutes As Integer = m_MinutesLength
        Dim Hours As Integer = Minutes \ 60
        Dim RestMinutes As Integer = Minutes - Hours * 60
        Dim LengthString As String = Hours.ToString.PadLeft(2, "0") & ":" & RestMinutes.ToString.PadLeft(2, "0")

        Return "Start: " & m_StartTime.ToShortTimeString & ", " & My.LanguageInfo.TextTimeLength & ": " & LengthString & DayString
    End Function

#End Region

    ' Liste mit Wochentagen, beginnend am Montag
    Private Shared m_Days As New List(Of DayOfWeek)
    ' Dictionary mit Wochentags-Abkrzungen
    Private Shared m_DayStrings As New Dictionary(Of DayOfWeek, String)
    Shared Sub New()

        ' Sonntag hat den Enumerationswert 0, soll aber am Ende der Liste stehen
        For I As Integer = 1 To 6
            m_Days.Add(CType(I, DayOfWeek))
        Next
        m_Days.Add(DayOfWeek.Sunday)

        Dim DtFi As System.Globalization.DateTimeFormatInfo = My.Application.Culture.DateTimeFormat

        ' Dictionary mit Wochentags-Abkrzungen fllen
        For Each Day As DayOfWeek In m_Days
            m_DayStrings.Add(Day, ", " & DtFi.GetShortestDayName(Day))
        Next
    End Sub

      

End Class
