''' <summary>
''' speichert eine MP3-Datei mit mehreren Abschnitten
''' </summary>
Public Class MP3FileSaverMultiple
    Inherits MP3FileSaver

#Region " Fields / Events "

    ''' <summary>
    ''' Info, ob die separaten MP3-Dateien eine fortlaufende Nummer im Dateinamen erhalten sollen
    ''' </summary>
    Private m_CreateFileNumbers As Boolean

    ''' <summary>
    ''' Info, ob ein ID3-Tag zur Datei hinzugefgt werden soll
    ''' </summary>
    Private m_AddID3Tag As Boolean

    ''' <summary>
    ''' Encoding zum Speichern von Text-Informationen
    ''' </summary>
    Private Shared m_encoding As System.Text.Encoding

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Shared Konstruktor
    ''' </summary>
    Shared Sub New()
        m_encoding = New System.Text.ASCIIEncoding
    End Sub

    ''' <summary>
    ''' Konstruktor
    ''' </summary>
    ''' <param name="File">MP3-Datei</param>
    ''' <param name="Path">Pfad, unter dem gespeichert werden soll</param>
    ''' <param name="ProgressForm">Referenz zur Fortschrittsanzeige-Form</param>
    ''' <param name="CreateFileNumbers">Info, ob die separaten MP3-Dateien eine fortlaufende Nummer im Dateinamen erhalten sollen</param>
    ''' <param name="AddID3Tag">Info, ob ein ID3-Tag zur Datei hinzugefgt werden soll</param>
    Public Sub New(ByVal File As MP3File, ByVal Path As String, ByVal ProgressForm As frm_Progress, ByVal CreateFileNumbers As Boolean, ByVal AddID3Tag As Boolean)
        MyBase.New(File, Path, ProgressForm)
        m_CreateFileNumbers = CreateFileNumbers
        m_AddID3Tag = AddID3Tag
    End Sub


#End Region

#Region " Methods "

    ''' <summary>
    ''' fhrt die Aktion zum Schreiben der MP3-Datei aus
    ''' </summary>
    Public Overrides Sub Execute()

        ' Liste mit kleingeschriebenen Dateinamen
        Dim LowerFileNames As New List(Of String)

        ' Liste mit tatschlich verwendeten MP3-Abschnitten
        Dim UsedSections As New List(Of MP3FileSection)

        ' Dictionary mit MP3-Abschnitten und vorgesehenem Dateinamen
        Dim FileNames As New Dictionary(Of MP3FileSection, String)

        ' Temporrer Pfad zur Ausgabe der einzelnen Dateien
        ' (es ist schneller, eine Datei auf der Festplatte zu erstellen
        ' und spter komplett auf die SD-Karte zu kopieren )
        Dim TempDirPath As String = IO.Path.GetTempPath & "\" & Guid.NewGuid.ToString
        IO.Directory.CreateDirectory(TempDirPath)

        Dim StartPosition As Long = 0
        Dim EndPosition As Long = 0

        ' Liste mit tatschlich verwendeten MP3-Abschnitten erstellen
        ' (kein Abschnitt darf die Lnge Null haben)
        For Each Section As MP3FileSection In Me.File
            If Section.Text.IndexOf("-") > -1 Then
                Dim Parts() As String = Section.Text.Split("-")
                If Parts.Length = 2 AndAlso Parts(0).Trim <> "" AndAlso Parts(1).Trim <> "" Then
                    EndPosition = Section.EndPosition
                    Dim Length As Long = EndPosition - StartPosition
                    If Length > 0L Then
                        UsedSections.Add(Section)
                    End If
                    StartPosition = EndPosition
                End If
            End If
        Next

        ' sptere Dateinamen ermitteln
        Dim Counter As Integer = 1
        If m_CreateFileNumbers Then
            ' Lnge der ggf. notwendigen fhrenden Nullen fr durchnumerierte Datei-Namen 
            Dim PadLength As Integer = UsedSections.Count.ToString.Length

            For Each Section As MP3FileSection In UsedSections
                FileNames.Add(Section, GetFreeFileName(LowerFileNames, Counter.ToString.PadLeft(PadLength, "0") & " - " & Section.Text.Trim, ".mp3"))
                Counter += 1
            Next
        Else
            For Each Section As MP3FileSection In UsedSections
                FileNames.Add(Section, GetFreeFileName(LowerFileNames, Section.Text.Trim, ".mp3"))
            Next
        End If

        ' Maximum der Fortschrittsanzeige im ProgressForm festlegen 
        Me.ProgressForm.RefreshMaxProgress(UsedSections.Count)
        Counter = 0
        StartPosition = 0


        ' Hier beginnt das Erstellen der einzelnen MP3-Dateien
        Try
            ' FileStream aus Quell-MP3-Datei erstellen
            Using Source As New IO.FileStream(MP3Source.OutputPath & "\" & Me.File.FileName & ".mp3", IO.FileMode.Open)

                ' Ein Byte-Array als Puffer zum Kopieren erstellen
                Dim Buffer(2047) As Byte
                Dim BufferLength As Integer


                ' jeden Datei-Abschnitt mit Titel als separate MP3-Datei kopieren
                For Each Section As MP3FileSection In UsedSections

                    ' Bei Click auf "Abbrechen"-Button Schleife verlassen
                    If Me.ProgressForm.CancelButtonPushed Then Exit For

                    EndPosition = Section.EndPosition
                    Counter += 1
                    ' Fortschrittsanzeige aktualisieren
                    Me.ProgressForm.RefreshProgress(Counter)
                    Me.ProgressForm.RefreshText(FileNames(Section))

                    ' Lnge der Datei in Bytes
                    Dim Length As Long = EndPosition - StartPosition


                    ' FileStream zum Schreiben der neuen MP3-Datei
                    Using FS As New IO.FileStream(TempDirPath & "\" & FileNames(Section), IO.FileMode.OpenOrCreate, IO.FileAccess.Write, IO.FileShare.Read)
                        While Length > 0
                            ' Anzahl der zu kopierenden Bytes ermitteln
                            If Length >= 2048 Then
                                BufferLength = 2048
                                Length -= 2048
                            Else
                                BufferLength = Length
                                Length = 0
                            End If
                            ' Bytes aus Quelldatei in Buffer schreiben
                            Source.Read(Buffer, 0, BufferLength)
                            ' Bytes aus Buffer in neue MP3-Datei schreiben
                            FS.Write(Buffer, 0, BufferLength)
                        End While
                        ' ggf. ID3-Tag hinzufgen
                        If m_AddID3Tag Then
                            AddID3Tag(FS, Section.Text)
                        End If
                        FS.Close()
                    End Using

                    ' Die eben erstellte einzelne MP3-Datei an den Zielpfad (z.B. auf SD-Karte) kopieren
                    IO.File.Copy(TempDirPath & "\" & FileNames(Section), Path & "\" & FileNames(Section))

                    ' Den Erstellungszeitpunkt fr die MP3-Datei so korrigieren, dass er der tatschlichen Empfangs-Zeit entspricht
                    Dim FI As New IO.FileInfo(Path & "\" & FileNames(Section))
                    Dim TmpTime As Date = Section.CreationTime
                    FI.CreationTime = TmpTime
                    FI.LastAccessTime = TmpTime
                    FI.LastWriteTime = TmpTime


                    StartPosition = Section.EndPosition
                Next

                Source.Close()
            End Using
        Catch ex As Exception
            Throw ex
        Finally
            ' Egal ob ein Fehler aufgetreten ist oder nicht,
            ' die temporren Dateien und das temporre Directory mssen zum Schluss gelscht werden
            Try
                Dim Files() As String = IO.Directory.GetFiles(TempDirPath)
                For Each txt As String In Files
                    Try
                        IO.File.Delete(txt)
                    Catch ex As Exception

                    End Try
                Next
            Catch ex As Exception

            End Try
            Try
                IO.Directory.Delete(TempDirPath, True)
            Catch ex As Exception

            End Try
        End Try

    End Sub


    ''' <summary>
    ''' fgt einen ID3V1-Tag am Ende des FileStreams einer zu schreibenden MP3-Datei an
    ''' </summary>
    ''' <param name="FS">FileStreams</param>
    ''' <param name="Title">Text mit Titel und Interpret</param>
    Private Sub AddID3Tag(ByVal FS As IO.FileStream, ByVal Title As String)

        Dim offset As Integer = 0
        Dim data(127) As Byte
        Dim temp() As Byte

        data(0) = CByte(84)
        ' T
        data(1) = CByte(65)
        ' A
        data(2) = CByte(71)
        ' G
        offset += 3

        Dim ArtistText As String = ""
        Dim TitleText As String = ""



        If Title.IndexOf(" - ") > -1 Then
            ArtistText = SharedFunctions.GetSubString(Title, "", " - ").Trim()
            TitleText = SharedFunctions.GetSubString(Title, " - ", "").Trim()
        End If


        ' title
        temp = GetBytes(TitleText, 30)
        Buffer.BlockCopy(temp, 0, data, offset, temp.Length)
        offset += 30

        ' artist
        temp = GetBytes(ArtistText, 30)
        Buffer.BlockCopy(temp, 0, data, offset, temp.Length)
        offset += 30

        ' album
        'temp = GetBytes(Me.MP3File.StationName, 30)
        'Buffer.BlockCopy(temp, 0, data, offset, temp.Length)
        offset += 30

        ' year
        'temp = GetBytes(Me.MP3File(0).CreationTime.Year.ToString, 4)
        'Buffer.BlockCopy(temp, 0, data, offset, temp.Length)
        offset += 4

        temp = GetBytes("WebRadioRecorder", 30)
        Buffer.BlockCopy(temp, 0, data, offset, temp.Length)
        'offset += 30

        data(127) = 12

        FS.Write(data, 0, data.Length)
    End Sub


    ''' <summary>
    ''' Liefert ein Array aus Bytes fr einen String mit Lngenbegrenzung
    ''' </summary>
    ''' <param name="Text">String</param>
    ''' <param name="MaxLength">Maximale Lnge</param>
    ''' <returns>Byte-Array</returns>
    Private Function GetBytes(ByVal Text As String, ByVal MaxLength As Integer) As Byte()
        If Text.Length > MaxLength Then
            Text = Text.Substring(0, MaxLength)
        End If
        Return m_encoding.GetBytes(Text)

    End Function


    ''' <summary>
    ''' liefert einen nicht vorhandenen Dateinamen, ggf. mit Erweiterung durch eine Ziffer
    ''' </summary>
    ''' <param name="KnownLowerFileNames">Liste mit bereits im Verzeichnis vorhandenen Dateinamen</param>
    ''' <param name="FileName">gewnschter Dateiname</param>
    ''' <param name="Extension">Erweiterung</param>
    ''' <returns>noch nicht vorhandener Dateiname</returns>
    Private Function GetFreeFileName(ByVal KnownLowerFileNames As List(Of String), ByVal FileName As String, ByVal Extension As String) As String
        FileName = ReplaceNoFileChars(FileName)
        Dim NewFileName As String = FileName & Extension
        Dim Counter As Integer = 1
        While KnownLowerFileNames.Contains(NewFileName.ToLower)
            NewFileName = FileName & Counter.ToString & Extension
            Counter += 1
        End While
        KnownLowerFileNames.Add(NewFileName.ToLower)

        Return NewFileName
    End Function

    ''' <summary>
    ''' ersetzt alle fr Dateinamen ungeeigneten Zeichen in einem String
    ''' </summary>
    ''' <param name="FileName">Original-Dateiname</param>
    Private Function ReplaceNoFileChars(ByVal FileName As String) As String
        Dim Result As String = FileName
        If FileName.Trim = "" Then FileName = "x"
        For Each txt As Char In IO.Path.GetInvalidFileNameChars
            Result = Result.Replace(txt.ToString, " ")
        Next

        While Result.IndexOf("  ") > -1
            Result = Result.Replace("  ", " ")
        End While

        Return Result
    End Function

#End Region

End Class
