Imports System.Xml
''' <summary>
''' Optionen zum Speichern der MP3-Dateien
''' </summary>
Public Class MP3SaveOptions

#Region " Fields / Events "

    ''' <summary>
    ''' Info, ob Speichern in separaten MP3-Dateien bevorzugt wird
    ''' </summary>
    Private m_SaveMultipleFilesPreferred As Boolean = True

    ''' <summary>
    ''' Info, ob Dateinamen eine fhrende Nummer erhalten sollen
    ''' </summary>
    Private m_EnumerateFiles As Boolean = False

    ''' <summary>
    ''' Info, ob separate Dateien einen ID3-Tag erhalten
    ''' </summary>
    Private m_AddID3Tag As Boolean = True

    ''' <summary>
    ''' zuletzt verwendetes Directory fr separate MP3-Dateien
    ''' </summary>
    Private m_LastSavedDirectory As String = ""

    ''' <summary>
    ''' zuletzt verwendetes Directory fr Speichern als einzelne MP3-Datei
    ''' </summary>
    Private m_LastSingleFileDirectory As String = ""

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Parameterloser Konstruktor
    ''' </summary>
    Public Sub New()

    End Sub

    ''' <summary>
    ''' Konstruktor zum Erstellen beim Lesen einer XML-Datei
    ''' </summary>
    Public Sub New(ByVal Nd As XmlNode)
        m_SaveMultipleFilesPreferred = CBool(Nd.Attributes("SaveMultipleFilesPreferred").Value)
        m_EnumerateFiles = CBool(Nd.Attributes("EnumerateFiles").Value)
        m_AddID3Tag = CBool(Nd.Attributes("AddID3Tag").Value)
        m_LastSavedDirectory = Nd.Attributes("LastSavedDirectory").Value
        m_LastSingleFileDirectory = Nd.Attributes("LastSingleFileDirectory").Value
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' zuletzt verwendetes Directory fr separate MP3-Dateien
    ''' </summary>
    Property LastSavedDirectory() As String
        Get
            Return m_LastSavedDirectory
        End Get
        Set(ByVal value As String)
            m_LastSavedDirectory = value
        End Set
    End Property

    ''' <summary>
    ''' zuletzt verwendetes Directory fr Speichern als einzelne MP3-Datei
    ''' </summary>
    Property LastSingleFileDirectory() As String
        Get
            Return m_LastSingleFileDirectory
        End Get
        Set(ByVal value As String)
            m_LastSingleFileDirectory = value
        End Set
    End Property

    ''' <summary>
    ''' legt fest oder gibt zurck, ob Speichern in separaten MP3-Dateien bevorzugt wird
    ''' </summary>
    Property SaveMultipleFilesPreferred() As Boolean
        Get
            Return m_SaveMultipleFilesPreferred
        End Get
        Set(ByVal value As Boolean)
            m_SaveMultipleFilesPreferred = value
        End Set
    End Property

    ''' <summary>
    ''' legt fest oder gibt zurck, ob Dateinamen eine fhrende Nummer erhalten sollen
    ''' </summary>
    Property EnumerateFiles() As Boolean
        Get
            Return m_EnumerateFiles
        End Get
        Set(ByVal value As Boolean)
            m_EnumerateFiles = value
        End Set
    End Property

    ''' <summary>
    ''' legt fest oder gibt zurck, ob separate Dateien einen ID3-Tag erhalten
    ''' </summary>
    Property AddID3Tag() As Boolean
        Get
            Return m_AddID3Tag
        End Get
        Set(ByVal value As Boolean)
            m_AddID3Tag = value
        End Set
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' schreibt den Inhalt in eine XML-Datei
    ''' </summary>
    Public Sub Write_XML(ByVal Writer As XmlTextWriter)
        Writer.WriteAttributeString("SaveMultipleFilesPreferred", m_SaveMultipleFilesPreferred.ToString)
        Writer.WriteAttributeString("EnumerateFiles", m_EnumerateFiles.ToString)
        Writer.WriteAttributeString("AddID3Tag", m_AddID3Tag.ToString)
        Writer.WriteAttributeString("LastSavedDirectory", m_LastSavedDirectory)
        Writer.WriteAttributeString("LastSingleFileDirectory", m_LastSingleFileDirectory)
    End Sub

#End Region

End Class
