Imports System.ComponentModel
''' <summary>
''' Control zum Anzeigen und ndern einer Uhrzeit
''' </summary>
<DefaultEvent("Changed"), DefaultProperty("Time")> _
Public Class ClockControl
    Inherits System.Windows.Forms.Control

#Region " Fields / Events "
    ' zentraler Punkt auf dem Control
    Private m_CentralPoint As PointF
    ' kreisfrmige Region zum Feststellen, ob die Maus sich im Control-Bereich befindet
    Private m_CircleRegion As System.Drawing.Region
    ' angezeigter Winkel fr den Stundenzeiger
    Private m_AngleHour As Integer = 0
    ' angezeigter Winkel fr den Minutenzeiger
    Private m_AngleMinute As Integer = 0

    ' aktuell mit der Maus bewegter Zeiger
    Private m_MovedHand As MovedHands = MovedHands.None
    ' der zuletzt mit der Maus eingestellte Zeiger-Winkel
    Private m_LastMovedAngle As Integer = -1

    ' Bitmap, die bei jedem Paint-Event gezeichnet wird
    Private m_BMP As Bitmap

    ' aktuell angezeigte Zeit
    Private m_Time As Date

    ''' <summary>
    ''' tritt beim ndern der Zeit von diesem Control aus auf
    ''' </summary>
    Public Event Changed()
#End Region

#Region " Properties "

    ''' <summary>
    ''' Legt die Uhrzeit fr ein Datum fest oder gibt das ausgewhlte Datum mit Uhrzeit zurck
    ''' </summary>
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
    Property Time() As Date
        Get
            Return m_Time
        End Get
        Set(ByVal value As Date)
            If m_Time <> value Then
                m_Time = value
                ' Winkel fr die Uhrzeiger neu ermitteln
                GetZeigerWerte()
                ' Neuzeichnen erzwingen
                m_BMP = Nothing
                Invalidate()
            End If
        End Set
    End Property

#End Region

#Region " Constructor "
    ''' <summary>
    ''' Konstruktor
    ''' </summary>
    Public Sub New()
        ' DoubleBuffering aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint, True)
        m_Time = Now
        ' Winkel zum Anzeigen der Uhrzeiger ermitteln
        GetZeigerWerte()
    End Sub
#End Region

#Region " Methods "
    ''' <summary>
    ''' berschreiben von OnResize, um den angezeigten Inhalt bei Grennderung des Controls zu aktualisieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)

        ' Mindestgre und quadratische Form erhalten
        If Me.Width < 80 Then Me.Width = 80
        If Me.Height < 80 Then Me.Height = 80
        Dim Max As Integer = Math.Max(Me.Width, Me.Height)
        Me.Width = Max
        Me.Height = Max

        ' zentralen Punkt ermitteln
        Dim Width As Single = Me.Width
        m_CentralPoint = New PointF(Width / 2.0F, Width / 2.0F)

        Dim gp As New System.Drawing.Drawing2D.GraphicsPath
        Dim RA As New RectangleF(0.0F, 0.0F, Width, Width)
        RA.Inflate(-2.0F, -2.0F)
        gp.AddEllipse(RA)

        ' kreisfrmige Region festlegen
        m_CircleRegion = New System.Drawing.Region(gp)
        ' Vollstndiges Neuzeichnen erzwingen
        m_BMP = Nothing
        Invalidate()
    End Sub
    ''' <summary>
    ''' berschreiben von OnPaint zum Zeichnen des Inhalts des Controls
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        ' hier wird nur auf eine Bitmap gezeichnet.
        ' Wenn vorhanden, wird die Bitmap gezeichnet, wenn nicht vorhanden, 
        ' wird diese neu erstellt

        If IsNothing(m_BMP) Then
            ' Neue Bitmap mit Abmessungen des Controls erstellen
            m_BMP = New Bitmap(Me.Width, Me.Height)
            ' Graphics-Object aus Bitmap erstellen, darauf kann gezeichnet werden
            Using g As Graphics = Graphics.FromImage(m_BMP)

                g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality

                Dim matrix As New Drawing2D.Matrix

                ' Hintergrund (weien Kreis mit schwarzem Rahmen) zeichnen
                Dim KreisRA As New RectangleF(0.0F, 0.0F, Me.Width, Me.Width)
                KreisRA.Inflate(-3.0F, -3.0F)
                g.FillEllipse(Brushes.White, KreisRA)
                Using ForePen As New Pen(Color.Black, 2.0F)
                    g.DrawEllipse(ForePen, KreisRA)
                End Using

                ' Ziffern an den notwendigen Positionen zeichnen 
                Dim ZPoints(0) As PointF
                For I As Integer = 1 To 12
                    matrix = New Drawing2D.Matrix
                    matrix.RotateAt(I * 30, m_CentralPoint, Drawing2D.MatrixOrder.Append)
                    ZPoints(0) = New PointF(m_CentralPoint.X, 15.0F)
                    matrix.TransformPoints(ZPoints)
                    Dim xSize As SizeF = g.MeasureString(I.ToString, Me.Font)
                    ZPoints(0).X -= xSize.Width / 2.0F
                    ZPoints(0).Y -= xSize.Height / 2.0F
                    g.DrawString(I.ToString, Me.Font, Brushes.Black, ZPoints(0))
                Next

                ' Zeiger zeichnen
                Using RedPen As New Pen(Color.Red, 5)
                    RedPen.EndCap = Drawing2D.LineCap.Round
                    RedPen.StartCap = Drawing2D.LineCap.Round

                    Dim LinePoints(1) As PointF

                    LinePoints(0) = New PointF(m_CentralPoint.X, 16.0F)
                    LinePoints(1) = New PointF(m_CentralPoint.X, m_CentralPoint.X + 4.0F)

                    ' Minutenzeiger
                    matrix = New Drawing2D.Matrix
                    matrix.RotateAt(m_AngleMinute, m_CentralPoint, Drawing2D.MatrixOrder.Append)
                    matrix.TransformPoints(LinePoints)

                    g.DrawLine(RedPen, LinePoints(0).X, LinePoints(0).Y, LinePoints(1).X, LinePoints(1).Y)

                    LinePoints(0) = New PointF(m_CentralPoint.X, 28.0F)
                    LinePoints(1) = New PointF(m_CentralPoint.X, m_CentralPoint.X + 4.0F)

                    ' Stundenzeiger
                    matrix = New Drawing2D.Matrix
                    matrix.RotateAt(m_AngleHour, m_CentralPoint, Drawing2D.MatrixOrder.Append)
                    matrix.TransformPoints(LinePoints)
                    g.DrawLine(RedPen, LinePoints(0).X, LinePoints(0).Y, LinePoints(1).X, LinePoints(1).Y)
                    ' an der Kreuzungsstelle der Zeiger kleinen weien Kreis zeichnen
                    g.DrawEllipse(Pens.White, New RectangleF(m_CentralPoint.X - 2.0F, m_CentralPoint.X - 2.0F, 4.0F, 4.0F))
                End Using
            End Using
        End If

        ' Die eben erstellte Bitmap wird auf dem Graphics-Object des Controls gezeichnet
        e.Graphics.DrawImage(m_BMP, 0, 0)
    End Sub

    ''' <summary>
    ''' liefert die anzuzeigenden Winkel fr die Uhrzeiger
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub GetZeigerWerte()
        Dim Minutenwinkel As Integer = m_Time.Minute * 6
        Dim ZStunde As Integer = m_Time.Hour
        If ZStunde > 11 Then
            ZStunde -= 12
        End If

        ' 30 Grad entspricht 60 Minuten
        ' -> 1 Min = 0,5 Grad

        Dim StundenWinkel As Integer = ZStunde * 30 + CInt(Math.Round(CSng(m_Time.Minute) / 2.0F, 0))

        m_AngleHour = StundenWinkel
        m_AngleMinute = Minutenwinkel
    End Sub

    ''' <summary>
    ''' berschreiben von OnMouseDown, um die Position der Zeiger zu ndern
    ''' </summary>
    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)
        If e.Button = MouseButtons.Left AndAlso m_CircleRegion.IsVisible(e.Location) Then
            ' den der MousePosition am nchsten liegenden Zeiger finden
            Dim Winkel As Double = GetMouseAngle(e.Location)

            Dim DistanceHour As Double = FindShortestAngleDistance(CDbl(m_AngleHour), Winkel)
            Dim DistanceMinute As Double = FindShortestAngleDistance(CDbl(m_AngleMinute), Winkel)

            If DistanceHour <= DistanceMinute Then
                m_MovedHand = MovedHands.Hour
            Else
                m_MovedHand = MovedHands.Minute
            End If
            MoveTo(CInt(Winkel))
        End If
    End Sub
    ''' <summary>
    ''' berschreiben von OnMouseUp, um auf weitere nderungen der Zeigerposition bei Mausbewegung 
    ''' zu verzichten
    ''' </summary>
    Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseUp(e)
        m_MovedHand = MovedHands.None
        m_LastMovedAngle = -1
    End Sub
    ''' <summary>
    ''' berschreiben von OnMouseMove, um die Positionen der Zeiger zu ndern
    ''' </summary>
    Protected Overrides Sub OnMouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseMove(e)
        If m_MovedHand = MovedHands.None Then Exit Sub
        MoveTo(GetMouseAngle(e.Location))
    End Sub


    ''' <summary>
    ''' bewegt den aktuell ausgewhlten Zeiger zu einem Ziel-Winkel
    ''' und legt die angezeigte Zeit neu fest
    ''' </summary>
    Private Sub MoveTo(ByVal Angle As Integer)
        If Angle = 360 Then Angle = 0
        If m_LastMovedAngle = Angle Then Exit Sub
        ' Feststellen, ob der Zeiger vor oder zuck bewegt wird
        Dim ActualAngle As Integer
        If m_MovedHand = MovedHands.Hour Then
            ActualAngle = m_AngleHour
        Else
            ActualAngle = m_AngleMinute
        End If
        Dim StepsForward, StepsBackward As Integer
        Dim ZValue As Integer = ActualAngle
        While ZValue <> Angle
            StepsForward += 1
            ZValue += 1
            If ZValue = 360 Then
                ZValue = 0
            End If
        End While

        ZValue = ActualAngle
        'rckwrts berechnen
        While ZValue <> Angle
            StepsBackward += 1
            ZValue -= 1
            If ZValue = -1 Then
                ZValue = 359
            End If
        End While


        Dim Stunde As Integer = m_Time.Hour
        Dim Minute As Integer = m_Time.Minute
        Dim AddDays As Integer = 0


        If m_MovedHand = MovedHands.Minute Then
            ' wenn der Minutenzeiger bewegt wird
            If StepsBackward < StepsForward Then
                ' zurck bewegen
                For I As Integer = 1 To StepsBackward \ 6
                    Minute -= 1
                    If Minute = -1 Then
                        Minute = 59
                        Stunde -= 1
                        If Stunde = -1 Then
                            Stunde = 23
                            AddDays = -1
                        End If
                    End If
                Next
            Else
                ' vorwrts bewegen
                For I As Integer = 1 To StepsForward \ 6
                    Minute += 1
                    If Minute = 60 Then
                        Minute = 0
                        Stunde += 1
                        If Stunde = 24 Then
                            Stunde = 0
                            AddDays = 1
                        End If
                    End If
                Next
            End If
        Else
            ' wenn der Stundenzeiger bewegt wird
            If StepsBackward < StepsForward Then
                ' zurck bewegen
                For I As Integer = 1 To StepsBackward \ 30
                    Stunde -= 1
                    If Stunde = -1 Then
                        Stunde = 23
                        AddDays = -1
                    End If
                Next
            Else
                ' vorwrts bewegen
                For I As Integer = 1 To StepsForward \ 30
                    Stunde += 1
                    If Stunde = 24 Then
                        Stunde = 0
                        AddDays = 1
                    End If
                Next
            End If
        End If

        ' Neu anzuzeigende Zeit ermitteln
        Dim NewTime As New Date(m_Time.Year, m_Time.Month, m_Time.Day, Stunde, Minute, m_Time.Second)
        NewTime = NewTime.AddDays(AddDays)

        ' Wenn die neu anzuzeigende Zeit gegenber der bereits angezeigten gendert ist
        If m_Time <> NewTime Then
            m_Time = NewTime
            ' Zeiger-Positionen ermitteln
            GetZeigerWerte()
            ' Control neu zeichnen
            m_BMP = Nothing
            Invalidate()
            ' nderungs-Event liefern
            RaiseEvent Changed()
        End If

        ' zuletzt eingestellten Winkel merken
        m_LastMovedAngle = Angle
    End Sub

    ''' <summary>
    ''' liefert einen Winkel fr eine Maus-Position innerhalb des Uhr-Kreises
    ''' </summary>
    ''' <param name="MousePosition">Point</param>
    ''' <returns>Winkel zwischen 0 und 360 Grad</returns>
    Private Function GetMouseAngle(ByVal MousePosition As Point) As Double
        Dim KoordinateMousePoint As New PointF(CSng(MousePosition.X) - m_CentralPoint.X, CSng(MousePosition.Y) - m_CentralPoint.X)
        ' Winkel im Bogenmass
        Dim AngleBM As Double = Math.Atan2(KoordinateMousePoint.Y, KoordinateMousePoint.X)

        Dim Angle As Double = AngleBM * 180.0R / Math.PI
        'Verschiebung um 90 Grad 
        Angle += 90.0R
        If Angle < 0 Then
            'so gibts nur positive Winkel 
            Angle += 360.0R
        End If
        Return Angle
    End Function


    ''' <summary>
    ''' liefert den krzesten Abstand zwischen zwei Winkeln innerhalb des Uhrkreises
    ''' </summary>
    Private Function FindShortestAngleDistance(ByVal Angle1 As Double, ByVal Angle2 As Double) As Double

        If Angle1 = Angle2 Then Return 0.0R

        Dim Result1, Result2 As Double

        If Angle1 > Angle2 Then
            Result1 = Angle1 - Angle2
        ElseIf Angle1 < Angle2 Then
            Result1 = Angle2 - Angle1
        End If

        Result2 = 360.0R - Result1
        If Result2 < Result1 Then
            Return Result2
        Else
            Return Result1
        End If
    End Function

#End Region

#Region " Enums "
    ''' <summary>
    ''' Enumeration fr bewegten Zeiger
    ''' </summary>
    ''' <remarks></remarks>
    Private Enum MovedHands
        None
        Hour
        Minute
    End Enum
#End Region

End Class
