''' <summary>
''' UserControl zum Anzeigen der vorhandenen MP3Files
''' </summary>
Public Class MP3FilePanel

#Region " Fields / Events "
    ' Sammlung verwendeter MP3-Dateien
    Private WithEvents m_MP3Source As MP3Source
    ' Optionen zum Speichern / Kopieren der MP3-Dateien
    Private m_MP3SaveOptions As MP3SaveOptions

    ' ausgewhlte MP3-Datei
    Private WithEvents m_SelectedFile As MP3File


    ' Delegat-Methode, die hier fr verschiedene Methoden bei Zugriff durch fremde Threads bentigt wird
    Private Delegate Sub MP3FileDelegate(ByVal File As MP3File)

    ' Delegat fr das Hinzufgen einer neuen MP3-Datei
    Private m_AddFileDelegate As MP3FileDelegate
    ' Delegat fr das Entfernen einer MP3-Datei
    Private m_RemoveFileDelegate As MP3FileDelegate
    ' Delegat fr das Auswhlen einer MP3-Datei
    Private m_SelectFileDelegate As MP3FileDelegate
    ' Delegat fr die nderung des Empfangsstatus der selektierten MP3-Datei
    Private m_ChangeReceivingDelegate As MP3FileDelegate

#End Region

#Region " Constructor "
    Public Sub New()

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Images fr TreeNodes in ImageList aufnehmen
        Me.ImageList1.Images.Add(My.Resources.GenericMusicDoc)
        Me.ImageList1.Images.Add(My.Resources.GenericMusicDocRecording)

        ' Delegaten fr Hinzufgen, Entfernen, Selektieren und Statusnderung von TreeNodes 
        ' von einem anderen Thread aus deklarieren
        m_AddFileDelegate = New MP3FileDelegate(AddressOf Me.AddFile)
        m_RemoveFileDelegate = New MP3FileDelegate(AddressOf Me.RemoveFile)
        m_SelectFileDelegate = New MP3FileDelegate(AddressOf Me.SelectFile)
        m_ChangeReceivingDelegate = New MP3FileDelegate(AddressOf Me.ChangeReceiving)

        Me.ColorLabel1.Text = My.LanguageInfo.TextRecords
        Me.ToolStripButtonDelete.Text = My.LanguageInfo.TextRemove
        Me.ToolStripButtonPlay.Text = My.LanguageInfo.TextPlay
        Me.ToolStripButtonSave.Text = My.LanguageInfo.TextSave
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Zuweisen der RadioRecorder-Daten ber Property
    ''' </summary>
    WriteOnly Property RadioRecorderSource() As RadioRecorderSource
        Set(ByVal value As RadioRecorderSource)
            If value IsNot Nothing Then
                m_MP3SaveOptions = value.MP3SaveOptions
                m_MP3Source = value.MP3Source


                ReadFiles()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Erstellt spezielle MP3FileTreeNodes fr alle MP3-Dateien in der MP3-Dateiliste
    ''' und zeigt diese im TreeView an
    ''' </summary>
    Private Sub ReadFiles()
        For Each File As MP3File In m_MP3Source
            Me.TreeView1.Nodes.Add(New MP3FileTreeNode(File))
        Next
        ' der oberste TreeNode wird selektiert
        Me.SelectFile(m_MP3Source.SelectedFile)
    End Sub

#End Region

#Region " Methods "

    ''' <summary>
    ''' tritt ein nach dem AfterSelect-Event des TreeView
    ''' </summary>
    Private Sub TreeView1_AfterSelect(ByVal sender As System.Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles TreeView1.AfterSelect
        Dim Nd As MP3FileTreeNode = TreeView1.SelectedNode
        m_MP3Source.SelectedFile = Nd.File
    End Sub

    ''' <summary>
    ''' Entfernt einen Treenode bei Entfernen der dazu gehrenden MP3-Datei
    ''' </summary>
    Private Sub RemoveFile(ByVal File As MP3File)
        ' Das Entfernen einer MP3-Datei geschieht ggf. in einem von diesem Control
        ' unabhngigen Thread. Also, dann diese Methode per Invoke aufrufen.
        If Me.InvokeRequired Then
            Me.Invoke(m_RemoveFileDelegate, New Object() {File})
            Exit Sub
        End If

        For Each Nd As MP3FileTreeNode In Me.TreeView1.Nodes
            If Nd.File Is File Then
                Nd.Remove()
                Exit For
            End If
        Next

    End Sub

    ''' <summary>
    ''' Passt ausgewhlten TreeNode und ToolStripButtons nach nderung
    ''' der selektierten MP3-Datei an
    ''' </summary>
    Private Sub SelectFile(ByVal File As MP3File)
        ' Das ndern der MP3-Datei-Auswahl geschieht ggf. in einem von diesem Control
        ' unabhngigen Thread. Also, dann diese Methode per Invoke aufrufen.
        If Me.InvokeRequired Then
            Me.Invoke(m_SelectFileDelegate, New Object() {File})
            Exit Sub
        End If

        If File Is Nothing Then
            ' wenn keine Datei ausgewhlt ist,
            ' alle Buttons deaktivieren
            Me.ToolStripButtonPlay.Enabled = False
            Me.ToolStripButtonDelete.Enabled = False
            Me.ToolStripButtonSave.Enabled = False
            Me.TreeView1.SelectedNode = Nothing
        Else
            For Each Nd As MP3FileTreeNode In Me.TreeView1.Nodes
                If Nd.File Is File Then
                    Me.TreeView1.SelectedNode = Nd
                    Nd.EnsureVisible()
                    Exit For
                End If
            Next
            ' es ist eine Datei ausgewhlt
            ' die Datei abgespielt werden kann immer
            Me.ToolStripButtonPlay.Enabled = True
            If File.Receiving Then
                ' Bei aktuellem Datei-Empfang wird das Lschen und Speichern verhindert
                Me.ToolStripButtonDelete.Enabled = False
                Me.ToolStripButtonSave.Enabled = False
            Else
                Me.ToolStripButtonDelete.Enabled = True
                Me.ToolStripButtonSave.Enabled = True
            End If
        End If

        m_SelectedFile = File
    End Sub

    ''' <summary>
    ''' Fgt einen TreeNode fr eine neu erstellte MP3-Datei hinzu
    ''' </summary>
    Private Sub AddFile(ByVal File As MP3File)
        ' Das Erstellen einer neuen MP3-Datei geschieht ggf. in einem von diesem Control
        ' unabhngigen Thread. Also, dann diese Methode per Invoke aufrufen.
        If Me.InvokeRequired Then
            Me.Invoke(m_AddFileDelegate, New Object() {File})
            Exit Sub
        End If

        ' Neuen MP3TreeNode erstellen
        Dim NewNode As New MP3FileTreeNode(File)
        Me.TreeView1.Nodes.Add(NewNode)
    End Sub

    ''' <summary>
    ''' Aktiviert / deaktiviert Buttons abhngig vom Empfangs-Status 
    ''' der selektierten MP3-Datei
    ''' </summary>
    Private Sub ChangeReceiving(ByVal File As MP3File)
        ' Das ndern des Empfangsstatus einer MP3-Datei geschieht ggf. in einem von diesem 
        ' Control unabhngigen Thread. Also, dann diese Methode per Invoke aufrufen.
        If Me.InvokeRequired Then
            Me.Invoke(m_ChangeReceivingDelegate, New Object() {File})
            Exit Sub
        End If

        If File.Receiving Then
            ' Bei aktuellem Datei-Empfang wird das Lschen und Speichern verhindert
            Me.ToolStripButtonDelete.Enabled = False
            Me.ToolStripButtonSave.Enabled = False
        Else
            Me.ToolStripButtonDelete.Enabled = True
            Me.ToolStripButtonSave.Enabled = True
        End If
    End Sub

    ''' <summary>
    ''' Spielt die selektierte Datei im MediaPlayer ab
    ''' </summary>
    Private Sub ToolStripButtonPlay_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonPlay.Click
        Process.Start(MP3Source.OutputPath & "\" & m_SelectedFile.FileName & ".mp3")
    End Sub

    ''' <summary>
    ''' Entfernt die selektierte MP3-Datei und den dazu gehrenden TreeNode
    ''' </summary>
    Private Sub ToolStripButtonDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonDelete.Click
        Dim SelFile As MP3File = m_SelectedFile
        If SelFile IsNot Nothing AndAlso (Not SelFile.Receiving) Then
            m_MP3Source.Remove(SelFile)
        End If
    End Sub

    ''' <summary>
    ''' Speichert die selektierte MP3-Datei unter einem anderen Pfad
    ''' </summary>
    Private Sub ToolStripButtonSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonSave.Click
        Dim frm As New frm_SaveMP3(m_MP3SaveOptions, m_SelectedFile)
        frm.ShowDialog()
    End Sub

    ''' <summary>
    ''' tritt ein bei Erstellen einer neuen MP3-Datei
    ''' </summary>
    Private Sub m_MP3Source_File_Created(ByVal File As MP3File) Handles m_MP3Source.File_Created
        AddFile(File)
    End Sub

    ''' <summary>
    ''' tritt ein bei Lschen einer neuen MP3-Datei
    ''' </summary>
    Private Sub m_MP3Source_File_Removed(ByVal File As MP3File) Handles m_MP3Source.File_Removed
        Me.RemoveFile(File)
    End Sub

    ''' <summary>
    ''' tritt ein bei ndern der selektierten MP3-Datei in der MP3-Datei-Liste
    ''' </summary>
    Private Sub m_MP3Source_SelectedFile_Changed() Handles m_MP3Source.SelectedFile_Changed
        Me.SelectFile(m_MP3Source.SelectedFile)
    End Sub

    ''' <summary>
    ''' tritt ein bei ndern des Empfangsstatus der selektierten MP3-Datei
    ''' </summary>
    Private Sub m_SelectedFile_Receiving_Changed() Handles m_SelectedFile.Receiving_Changed
        Me.ChangeReceiving(m_SelectedFile)
    End Sub

#End Region

End Class
