''' <summary>
''' TreeNode, der ein MP3File-Objekt referenziert und dessen Inhalt anzeigt
''' </summary>
Public Class MP3FileTreeNode
    Inherits TreeNode

#Region " Fields / Events / Delegates"

    ' Referenz zur MP3-Datei
    Private WithEvents m_File As MP3File

    ' Delegat-Methode fr Zugriff durch fremde Threads 
    Private Delegate Sub ChangedDelegate()
    ' Delegat zum Anzeigen von nderungen der MP3-Datei
    Private m_ChangedDelegate As ChangedDelegate

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor mit existierender MP3-Datei
    ''' </summary>
    Public Sub New(ByVal File As MP3File)
        m_File = File

        m_ChangedDelegate = New ChangedDelegate(AddressOf Me.RefreshNode)

        ' Inhalt der MP3-Datei anzeigen
        RefreshNode()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' liefert die referenzierte MP3-Datei
    ''' </summary>
    ReadOnly Property File() As MP3File
        Get
            Return m_File
        End Get
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' zeigt den Inhalt der referenzierten MP3-Datei neu an
    ''' </summary>
    Public Sub RefreshNode()

        If Me.TreeView IsNot Nothing AndAlso Me.TreeView.InvokeRequired Then
            Me.TreeView.Invoke(m_ChangedDelegate)
            Exit Sub
        End If

        ' Text fr den TreeNode bernehmen
        Me.Text = m_File.Text
        ' ImageIndex / SelectedImageIndex fr den TreeNode festlegen
        If m_File.Receiving Then
            Me.ImageIndex = 1
        Else
            Me.ImageIndex = 0
        End If
        Me.SelectedImageIndex = Me.ImageIndex
    End Sub

    ''' <summary>
    ''' tritt ein bei nderungen am Empfangsstatus der angezeigten MP3-Datei
    ''' </summary>
    Private Sub m_File_Receiving_Changed() Handles m_File.Receiving_Changed
        RefreshNode()
    End Sub

    ''' <summary>
    ''' tritt ein bei nderungen am Text der angezeigten MP3-Datei
    ''' </summary>
    Private Sub m_File_Text_Changed() Handles m_File.Text_Changed
        RefreshNode()
    End Sub

#End Region

End Class
