''' <summary>
''' UserControl zum Anzeigen der vorhandenen RadioStations
''' </summary>
Public Class RadioStationPanel

#Region " Fields / Events / Delegates "

    ' Quellen-Liste fr Radio-Stationen
    Private WithEvents m_RadioStations As RadioStationSource
    ' aktuell ausgewhlte Station
    Private WithEvents m_SelectedStation As RadioStation

    ' Delegat-Methode fr Zugriff durch fremde Threads
    Private Delegate Sub Recording_ChangedDelegate()
    ' Delegat zum Anzeigen von nderungen des Aufzeichnungs-Status der selektierten Radiostation
    Private m_Recording_ChangedDelegate As Recording_ChangedDelegate

#End Region

#Region " Constructor "
    Public Sub New()

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' ImageList fr TreeView mit geeigneten Images fllen

        Me.ImageList1.Images.Add(My.Resources.radiostation)
        Me.ImageList1.Images.Add(My.Resources.radiostationRecording)
        ' Delegat fr ReceivingChanged erstellen
        m_Recording_ChangedDelegate = New Recording_ChangedDelegate(AddressOf Me.OnRecording_Changed)
        Me.ColorLabel1.Text = My.LanguageInfo.TextStations
        Me.ToolStripButtonAdd.Text = My.LanguageInfo.TextAdd
        Me.ToolStripButtonPlay.Text = My.LanguageInfo.TextPlay
        Me.ToolStripButtonRecord.Text = My.LanguageInfo.TextRecord
        Me.ToolStripButtonRemove.Text = My.LanguageInfo.TextRemove
        Me.ToolStripButtonTasks.Text = My.LanguageInfo.TextTasks
    End Sub

#End Region

#Region " Properties "
    ''' <summary>
    ''' Zuweisen der Radiostationsliste ber Property
    ''' </summary>
    WriteOnly Property RadioStations() As RadioStationSource
        Set(ByVal value As RadioStationSource)
            m_RadioStations = value
            ' Stationen anzeigen
            ReadSource()
        End Set
    End Property
#End Region

#Region " Methods "

    ''' <summary>
    ''' Zeigt die vorhandenen Radiostationen als TreeNodes an
    ''' </summary>
    Private Sub ReadSource()
        Me.TreeView1.Nodes.Clear()
        For Each Station As RadioStation In m_RadioStations
            Dim NewNode As New RadioStationTreeNode(Station)
            Me.TreeView1.Nodes.Add(NewNode)
        Next
        Me.SelectStation(m_RadioStations.SelectedStation)
    End Sub

    ''' <summary>
    ''' tritt ein nach dem AfterSelect-Event des TreeView
    ''' </summary>
    Private Sub TreeView1_AfterSelect(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles TreeView1.AfterSelect
        Dim SelNode As RadioStationTreeNode = TreeView1.SelectedNode
        m_RadioStations.SelectedStation = SelNode.RadioStation
    End Sub


    ''' <summary>
    ''' Start oder Stop fr eine manuelle Aufzeichnung
    ''' </summary>
    Private Sub ToolStripButtonStartStopRecording_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRecord.Click
        Dim Station As RadioStation = m_RadioStations.SelectedStation
        If Station IsNot Nothing Then
            Station.IsRecording = Not Station.IsRecording
        End If
    End Sub

    ''' <summary>
    ''' spielt die gerade durch die RadioStation aufgenommene Datei im MediaPlayer ab
    ''' </summary>
    Private Sub ToolStripButtonPlay_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonPlay.Click
        Dim Station As RadioStation = m_RadioStations.SelectedStation
        If Station IsNot Nothing Then
            Process.Start(Station.URL)
        End If
    End Sub

    ''' <summary>
    ''' zeigt eine Windows Form zum Bearbeiten der Rekorder-Aufgaben fr den ausgewhlten Sender
    ''' </summary>
    Private Sub ToolStripButtonTasks_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonTasks.Click
        Dim Station As RadioStation = m_RadioStations.SelectedStation
        If Station IsNot Nothing Then
            Dim Frm As New frm_TaskEditor(Station)
            If Frm.ShowDialog = DialogResult.OK Then
                Station.RecordingTasks.SetTasks(Frm.RecordingTasks.ToArray)
                If Station.RecordingTasks.Count > 0 Then
                    Me.ToolStripButtonTasks.Image = My.Resources.RecordingTasksExisting
                Else
                    Me.ToolStripButtonTasks.Image = My.Resources.RecordingTask
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' tritt ein bei nderung der selektierten Radiostation in der Radiostations-Liste
    ''' </summary>
    Private Sub m_RadioStations_SelectedStation_Changed() Handles m_RadioStations.SelectedStation_Changed
        SelectStation(m_RadioStations.SelectedStation)
    End Sub

    ''' <summary>
    ''' Aktiviert und deaktiviert das Aussehen von Buttons bei nderung der selektierten Radiostation 
    ''' </summary>
    Private Sub SelectStation(ByVal Station As RadioStation)
        If Station Is Nothing Then
            Me.ToolStripButtonRemove.Enabled = False
            Me.ToolStripButtonRecord.Enabled = False
            Me.ToolStripButtonPlay.Enabled = False
            Me.ToolStripButtonTasks.Enabled = False
            Me.ToolStripButtonTasks.Image = My.Resources.RecordingTask
            Me.ToolStripButtonRecord.Image = My.Resources.RecordHS
            Me.ToolStripButtonRecord.Text = My.LanguageInfo.TextRecord
            Me.TreeView1.SelectedNode = Nothing
        Else

            For Each Nd As RadioStationTreeNode In Me.TreeView1.Nodes
                If Nd.RadioStation Is Station Then
                    Me.TreeView1.SelectedNode = Nd
                    Nd.EnsureVisible()
                    Exit For
                End If
            Next

            Me.ToolStripButtonRecord.Enabled = True
            Me.ToolStripButtonRemove.Enabled = True
            Me.ToolStripButtonPlay.Enabled = True
            Me.ToolStripButtonTasks.Enabled = True
            If Station.RecordingTasks.Count > 0 Then
                Me.ToolStripButtonTasks.Image = My.Resources.RecordingTasksExisting
            Else
                Me.ToolStripButtonTasks.Image = My.Resources.RecordingTask
            End If
            If Station.IsRecording Then
                Me.ToolStripButtonRecord.Image = My.Resources.PauseHS
                Me.ToolStripButtonRecord.Text = My.LanguageInfo.TextStop
            Else
                Me.ToolStripButtonRecord.Image = My.Resources.RecordHS
                Me.ToolStripButtonRecord.Text = My.LanguageInfo.TextRecord
            End If

        End If
        m_SelectedStation = Station
    End Sub

    ''' <summary>
    ''' Nach dem Erstellen einer neuen Radio-Station in der RadioStations-Liste
    ''' wird diese als selektierter TreeNode angezeigt
    ''' </summary>
    Private Sub m_RadioStations_StationAdded(ByVal NewStation As RadioStation) Handles m_RadioStations.StationAdded
        Dim NewNode As New RadioStationTreeNode(NewStation)
        Me.TreeView1.Nodes.Add(NewNode)
    End Sub

    ''' <summary>
    ''' Nach dem Entfernen einer Radio-Station aus der RadioStations-Liste
    ''' wird der dazu gehrende TreeNode entfernt
    ''' </summary>
    Private Sub m_RadioStations_StationRemoved(ByVal Station As RadioStation) Handles m_RadioStations.StationRemoved
        For Each Nd As RadioStationTreeNode In Me.TreeView1.Nodes
            If Nd.RadioStation Is Station Then
                Nd.Remove()
                Exit For
            End If
        Next
    End Sub

    ''' <summary>
    ''' Entfernt eine RadioStation und den dazu gehrenden TreeNode aus den vorhandenen Auflistungen
    ''' </summary>
    Private Sub ToolStripButtonRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRemove.Click
        If MessageBox.Show(My.LanguageInfo.QuestionRemoveSelectedStation, "Shoutcast Recorder", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
            Dim Node As RadioStationTreeNode = Me.TreeView1.SelectedNode
            m_RadioStations.Remove(Node.RadioStation)
        End If
    End Sub

    ''' <summary>
    ''' Fgt neue Radio-Station zur Auflistung hinzu
    ''' </summary>
    Private Sub ToolStripButtonAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAdd.Click
        Dim Txt As String = InputBox(My.LanguageInfo.QuestionInputStationURL, "Shoutcast Recorder")
        Txt = Txt.Trim
        If Txt <> "" Then
            m_RadioStations.AddStation(Txt)
        End If
    End Sub

    ''' <summary>
    ''' tritt ein bei nderung des Empfangs-Status der selektierten Radiostation
    ''' </summary>
    Private Sub m_SelectedStation_Recording_Changed() Handles m_SelectedStation.Recording_Changed
        OnRecording_Changed()
    End Sub

    ''' <summary>
    ''' Passt das Erscheinungsbild des Record-Buttons abhngig vom Empfangsstatus der
    ''' selektierten Radiostation an
    ''' </summary>
    Private Sub OnRecording_Changed()
        If Me.InvokeRequired Then
            Me.Invoke(m_Recording_ChangedDelegate)
            Exit Sub
        End If

        If m_SelectedStation.IsRecording Then
            Me.ToolStripButtonRecord.Image = My.Resources.PauseHS
            Me.ToolStripButtonRecord.Text = My.LanguageInfo.TextStop
        Else
            Me.ToolStripButtonRecord.Image = My.Resources.RecordHS
            Me.ToolStripButtonRecord.Text = My.LanguageInfo.TextRecord
        End If
    End Sub

#End Region

End Class
