''' <summary>
''' TreeNode, der ein RadioStation-Objekt referenziert und dessen Inhalt anzeigt
''' </summary>
Public Class RadioStationTreeNode
    Inherits TreeNode

#Region " Fields / Events / Delegates "

    ' referenzierte Radiostation
    Private WithEvents m_RadioStation As RadioStation

    ' Delegat-Methode fr Zugriff durch fremde Threads
    Private Delegate Sub ChangedDelegate()

    ' Delegat zum Anzeigen von nderungen des Aufzeichnungs-Status der referenzierten Radiostation
    Private m_RecordingChangedDelegate As ChangedDelegate
    ' Delegat zum Anzeigen von nderungen des Texts der referenzierten Radiostation
    Private m_TextChangedDelegate As ChangedDelegate

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor mit existierendem RadioStation-Objekt
    ''' </summary>
    Public Sub New(ByVal Station As RadioStation)
        m_RadioStation = Station
        ' Delegaten erstellen
        m_RecordingChangedDelegate = New ChangedDelegate(AddressOf Me.RefreshNodeRecording)
        m_TextChangedDelegate = New ChangedDelegate(AddressOf Me.RefreshNodeText)
        ' Inhalt des RadioStation-Objekts anzeigen
        RefreshNodeRecording()
        RefreshNodeText()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt das referenzierte RadioStation-Objekt zurck
    ''' </summary>
    ReadOnly Property RadioStation() As RadioStation
        Get
            Return m_RadioStation
        End Get
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' zeigt den Inhalt des RadioStation-Objekts neu an
    ''' </summary>
    ''' <remarks>ggf. notwendiges Invoke erfolgt in Parent-Klassen</remarks>
    Private Sub RefreshNodeRecording()

        If Me.TreeView IsNot Nothing AndAlso Me.TreeView.InvokeRequired Then
            Me.TreeView.Invoke(m_RecordingChangedDelegate)
            Exit Sub
        End If

        ' Text fr den TreeNode bernehmen
        Me.Text = m_RadioStation.Text
        ' ImageIndex / SelectedImageIndex fr den TreeNode festlegen
        If m_RadioStation.IsRecording Then
            Me.ImageIndex = 1
            Me.SelectedImageIndex = 1
        Else
            Me.ImageIndex = 0
            Me.SelectedImageIndex = 0
        End If
    End Sub

    Private Sub RefreshNodeText()

        If Me.TreeView IsNot Nothing AndAlso Me.TreeView.InvokeRequired Then
            Me.TreeView.Invoke(m_TextChangedDelegate)
            Exit Sub
        End If

        ' Text fr den TreeNode bernehmen
        Me.Text = m_RadioStation.Text

    End Sub

    ''' <summary>
    ''' tritt ein beim Recording_Changed-Event des referenzierten RadioStation-Objekts
    ''' </summary>
    Private Sub m_RadioStation_Recording_Changed() Handles m_RadioStation.Recording_Changed
        RefreshNodeRecording()
    End Sub

    ''' <summary>
    ''' tritt ein beim Text_Changed-Event des referenzierten RadioStation-Objekts
    ''' </summary>
    Private Sub m_RadioStation_Text_Changed() Handles m_RadioStation.Text_Changed
        RefreshNodeText()
    End Sub

#End Region

End Class

