''' <summary>
''' Windows Form zum Bearbeiten der Aufzeichnungs-Aufgaben fr eine Radiostation
''' </summary>
Public Class frm_TaskEditor

#Region " Fields / Events "
    ' aktuell ausgewhlte Aufzeichnungs-Aufgabe
    Private m_SelectedTask As RecordingTask
#End Region

#Region " Constructor "

    ''' <summary>
    ''' Konstruktor zum Anzeigen der Aufgaben fr eine Radio-Station
    ''' </summary>
    Public Sub New(ByVal Station As RadioStation)

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Images zur imageList hinzufgen
        Me.ImageList1.Images.Add(My.Resources.RecordingTask)
        Me.ImageList1.Images.Add(My.Resources.RecordingTaskRegularly)

        ' Beschriftung
        Me.ColorLabel1.Text = My.LanguageInfo.TextTasks1
        Me.Text = My.LanguageInfo.TextTasksFor & " " & Station.StationName
        Me.ToolStripButtonRemove.Text = My.LanguageInfo.TextRemove
        Me.ToolStripButtonAddPeriodicTask.Text = My.LanguageInfo.TextAddPeriodicTask
        Me.ToolStripButtonAddSingleTask.Text = My.LanguageInfo.TextAddSingleTask
        Me.ButtonCancel.Text = My.LanguageInfo.TextCancel

        ' TreeView mit TreeNodes fllen
        For Each Task As RecordingTask In Station.RecordingTasks
            Me.TreeView1.Nodes.Add(New RecordingTaskTreeNode(Task.CreateClone))
        Next
        ' ersten Node selektieren
        If Me.TreeView1.Nodes.Count > 0 Then
            Me.TreeView1.SelectedNode = Me.TreeView1.Nodes(0)
        End If
    End Sub
#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt eine Liste mit enthaltenen Aufzeichnungs-Aufgaben zurck
    ''' </summary>
    ReadOnly Property RecordingTasks() As List(Of RecordingTask)
        Get
            Dim Result As New List(Of RecordingTask)
            For Each Nd As RecordingTaskTreeNode In Me.TreeView1.Nodes
                Result.Add(Nd.Task)
            Next
            Return Result
        End Get
    End Property
#End Region

#Region " Methods "
   
    ''' <summary>
    ''' nach Auswahl eines TreeNodes dessen Task zur aktuellen machen und 
    ''' Details zu diesem Auftrag anzeigen
    ''' </summary>
    Private Sub TreeView1_AfterSelect(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles TreeView1.AfterSelect
        Dim Nd As RecordingTaskTreeNode = TreeView1.SelectedNode
        Me.SetSelectedTask(Nd.Task)
    End Sub

    ''' <summary>
    ''' nndert Buttons und angezeigte Controls fr den ausgewhlten Recorder-Auftrag
    ''' </summary>
    Private Sub SetSelectedTask(ByVal NewTask As RecordingTask)
        If m_SelectedTask IsNot NewTask Then
            m_SelectedTask = NewTask
            Me.PanelDetails.Controls.Clear()
            If m_SelectedTask Is Nothing Then
                Me.ToolStripButtonRemove.Enabled = False
            Else
                Dim Ctl As Control = m_SelectedTask.EditControl
                Me.PanelDetails.Controls.Add(Ctl)
                Ctl.Dock = DockStyle.Fill
                Me.ToolStripButtonRemove.Enabled = True
            End If
        End If
    End Sub

    ''' <summary>
    ''' Entfernen eines Recorder-Auftrags
    ''' </summary>
    Private Sub ToolStripButtonRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonRemove.Click
        Dim Idx As Integer = Me.TreeView1.Nodes.IndexOf(Me.TreeView1.SelectedNode)
        If Idx > 0 Then
            Idx -= 1
        End If
        Me.TreeView1.Nodes.Remove(Me.TreeView1.SelectedNode)
        If Me.TreeView1.Nodes.Count > 0 Then
            Me.TreeView1.SelectedNode = Me.TreeView1.Nodes(Idx)
        Else
            SetSelectedTask(Nothing)
        End If
    End Sub

    ''' <summary>
    ''' Erstellt einen Recorder-Auftrag fr eine einmalige Aufzeichnung 
    ''' </summary>
    Private Sub ToolStripButtonAddSingleTask_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAddSingleTask.Click
        Dim NewTask As New RecordingTaskSingle
        Dim NewNode As New RecordingTaskTreeNode(NewTask)
        Me.TreeView1.Nodes.Add(NewNode)
        Me.TreeView1.SelectedNode = NewNode
    End Sub

    ''' <summary>
    ''' Erstellt einen Recorder-Auftrag fr regelmige Aufzeichnung 
    ''' </summary>
    Private Sub ToolStripButtonAddPeriodicTask_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAddPeriodicTask.Click
        Dim NewTask As New RecordingTaskPeriodic
        Dim NewNode As New RecordingTaskTreeNode(NewTask)
        Me.TreeView1.Nodes.Add(NewNode)
        Me.TreeView1.SelectedNode = NewNode
    End Sub

#End Region

End Class